/*
 * Created on Aug 27, 2004
 * WindowFrame.java
 * 
 * Project: eyetrack_cvs
 */

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import pupil.PupilSessionHandler;
import session.SessionHandler;

import video.MYQTSession;

/**
 * @author chris the "main" class.
 */
public class WindowFrame extends JFrame {

	/** our session handler */
	private SessionHandler sessionHandler;

	/** the menu bar menus... */
	private JMenu[] menus;

	/** the menu menuItems ... */
	private JMenuItem[][] menuItems;

	/** flag indicating whether quicktime was successful started or not* */
	private boolean hasQuicktime = false;

	private File lastDirectory = new File("c:/bailey/research/projects/attention-management/tools/TAPRAV/data");

	/**
	 * default constructor
	 * 
	 * @param title
	 *            the frame title
	 */
	WindowFrame(String title) {
		super(title);

		//set up the quicktime environment
		try {
			MYQTSession.open();
			hasQuicktime = true;
		} catch (UnsatisfiedLinkError e) {
			//can't get quicktime started! give a warning!
			JOptionPane
					.showMessageDialog(
							this,
							"<html>Unable to start QuickTime Environment.<br>QuickTime for Java must be installed to view video content."
									+ "<br><br>Instructions for installing QuickTime for Java can be found here:<br>http://developer.apple.com/quicktime/qtjava/installation.html",
							"Could Not Start QuickTime",
							JOptionPane.WARNING_MESSAGE);
		} catch (Exception e) {
			//can't get quicktime started! give a warning!
			JOptionPane
					.showMessageDialog(
							this,
							"<html>Unable to start QuickTime Environment.<br>QuickTime for Java must be installed to view video content."
									+ "<br><br>Instructions for installing QuickTime for Java can be found here:<br>http://developer.apple.com/quicktime/qtjava/installation.html",
							"Could Not Start QuickTime",
							JOptionPane.WARNING_MESSAGE);
		}

		//set up the session handler
		sessionHandler = new PupilSessionHandler(this);
		setContentPane(sessionHandler);

		//set up the menubar
		buildMenuBar();
		setEnableDisableMenus();

		pack();

		//the window can be closed
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				cleanUpAndExit();
			}
		});

		//show to the world...
		this.setSize((int) ((1 / 1.65) * java.awt.Toolkit.getDefaultToolkit()
				.getScreenSize().width), (int) ((1 / 1.65) * java.awt.Toolkit
				.getDefaultToolkit().getScreenSize().height));
		setVisible(true);
	}

	/**
	 * method enables/disables menu menuItems relevant to an active view. Should
	 * be called after any event that may affect menu menuItems
	 */
	private void setEnableDisableMenus() {

		//do we have pupil data?
		if (((PupilSessionHandler) sessionHandler).havePupilData()) {
			menuItems[1][5].setEnabled(true);
			menuItems[3][0].setEnabled(true);
			menuItems[3][1].setEnabled(true);

		} else {
			menuItems[1][5].setEnabled(false);
			menuItems[3][0].setEnabled(false);
			menuItems[3][1].setEnabled(false);

		}

		//do we have task model data?
		if (((PupilSessionHandler) sessionHandler).haveTaskData())
			menuItems[1][6].setEnabled(true);
		else
			menuItems[1][6].setEnabled(false);

		//do we have video?
		if (((PupilSessionHandler) sessionHandler).haveVideoData()
				&& hasQuicktime)
			menuItems[3][4].setEnabled(true);
		else
			menuItems[3][4].setEnabled(false);

		//do we have any data?
		if (((PupilSessionHandler) sessionHandler).havePupilData()
				|| ((PupilSessionHandler) sessionHandler).haveTaskData()) {
			//focus stuff
			menuItems[1][1].setEnabled(true);
			menuItems[1][2].setEnabled(true);
			menuItems[1][3].setEnabled(true);

			//selection stuff
			menuItems[2][0].setEnabled(true);
		} else {
			//focus stuff
			menuItems[1][1].setEnabled(false);
			menuItems[1][2].setEnabled(false);
			menuItems[1][3].setEnabled(false);

			//selection stuff
			menuItems[2][0].setEnabled(false);
		}

		//how many perspectives?
		if (sessionHandler.getNumberOfViews() > 1) {
			menuItems[0][3].setEnabled(true);
		} else {
			menuItems[0][3].setEnabled(false);
		}
	}

	/**
	 * method builds the menu bar
	 *  
	 */
	private void buildMenuBar() {
		JMenuBar mb = new JMenuBar();

		menus = new JMenu[4];
		menuItems = new JMenuItem[menus.length][];

		//File menu
		menus[0] = new JMenu("File");
		menus[0].setMnemonic(KeyEvent.VK_F);
		menuItems[0] = new JMenuItem[12];

		//New Session
		menuItems[0][0] = new JMenuItem("New Session");
		menuItems[0][0].setMnemonic(KeyEvent.VK_N);
		menuItems[0][0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
				KeyEvent.CTRL_MASK));
		menuItems[0][0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//simply present a blank session
				sessionHandler.purge();
				sessionHandler.repaint();

				//enable view relevent menus
				setEnableDisableMenus();

			}
		});
		menus[0].add(menuItems[0][0]);

		//New View Tab
		menuItems[0][1] = new JMenuItem("New View Tab");
		menuItems[0][1].setMnemonic(KeyEvent.VK_T);
		menuItems[0][1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
				KeyEvent.CTRL_MASK));
		menuItems[0][1].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//upon menu item selection, add a new default
				//view
				sessionHandler.addView();

				setEnableDisableMenus();
			}

		});
		menus[0].add(menuItems[0][1]);

		//Open Session
		menuItems[0][2] = new JMenuItem("Open Session");
		menuItems[0][2].setMnemonic(KeyEvent.VK_O);
		menuItems[0][2].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				KeyEvent.CTRL_MASK));
		menuItems[0][2].addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				//on this menu select, user should be presented with
				//a session file to open

				//get a file explorer...
				JFileChooser fc = new JFileChooser();
				fc.addChoosableFileFilter(new EyeTrackFileFilter("ses",
						"Session File Type"));

				int returnVal = fc.showOpenDialog(WindowFrame.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();

					//attempt to read file
					try {
						sessionHandler.readDataFromFile(file);
					} catch (Exception e) {
						JOptionPane.showMessageDialog(WindowFrame.this,
								"<html>Sorry, unable to read <br>"
										+ file.getAbsolutePath(),
								"Cannot Open File", JOptionPane.ERROR_MESSAGE);
						return;
					}

					//get the pupil context menu from the session handler
					JMenuItem[] pupilItems = ((PupilSessionHandler) sessionHandler)
							.getPupilTrialMenuItems();
					if (pupilItems != null && pupilItems.length != 0) {
						WindowFrame.this.menuItems[1][5].removeAll();
						for (int i = 0; i < pupilItems.length; i++)
							WindowFrame.this.menuItems[1][5].add(pupilItems[i]);
					}

					//and the task model context menu menuItems...
					JMenuItem[] taskItems = ((PupilSessionHandler) sessionHandler)
							.getTaskMenuItems();
					if (taskItems != null && taskItems.length != 0) {
						WindowFrame.this.menuItems[1][6].removeAll();
						for (int i = 0; i < taskItems.length; i++)
							WindowFrame.this.menuItems[1][6].add(taskItems[i]);
					}

					//reassess menus
					setEnableDisableMenus();
				}
			}
		});
		menus[0].add(menuItems[0][2]);

		//Close View Tab
		menuItems[0][3] = new JMenuItem("Close View Tab");
		menuItems[0][3].setMnemonic(KeyEvent.VK_C);
		menuItems[0][3].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
				KeyEvent.CTRL_MASK));
		menuItems[0][3].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//check that this is even possible...
				switch (sessionHandler.getNumberOfViews()) {
				case 0:
					//do nothing...
					return;
				case 1:
					//also do nothing
					return;
				default:
					sessionHandler.removeView();
				}
				setEnableDisableMenus();
			}
		});
		menus[0].add(menuItems[0][3]);

		//Rename View Tab
		menuItems[0][4] = new JMenuItem("Rename View Tab");
		menuItems[0][4].setMnemonic(KeyEvent.VK_R);
		menuItems[0][4].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
				KeyEvent.CTRL_MASK));
		menuItems[0][4].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//present dialog with current view's name. allow the user
				//to change and set this name
				String newName = (String) JOptionPane.showInputDialog(
						WindowFrame.this, "Set the name of the View",
						"Rename View", JOptionPane.PLAIN_MESSAGE, null, null,
						sessionHandler.getSelectedViewName());

				//If a string was returned, set the name.
				if ((newName != null) && (newName.length() > 0)) {
					sessionHandler.setSelectedViewName(newName);
				}

			}
		});
		menus[0].add(menuItems[0][4]);

		menus[0].addSeparator();

		//Save Session
		menuItems[0][5] = new JMenuItem("Save Session");
		menuItems[0][5].setMnemonic(KeyEvent.VK_S);
		menuItems[0][5].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				KeyEvent.CTRL_MASK));
		menuItems[0][5].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				try {
					sessionHandler.writeDataToFile();
				} catch (Exception e) {
					//if file doesn't exist, present save as dialog
					if (e.getMessage().equals("No File")) {
						//get a file explorer...
						JFileChooser fc = new JFileChooser();
						fc.addChoosableFileFilter(new EyeTrackFileFilter("ses",
								"Session File Type"));
						int returnVal = fc.showSaveDialog(WindowFrame.this);
						if (returnVal == JFileChooser.APPROVE_OPTION) {
							File file = fc.getSelectedFile();
							try {
								sessionHandler.writeDataToFile(file);
								return;
							} catch (Exception e1) {
								//fall through to error dialog...
							}
						} else
							return;
					}

					JOptionPane
							.showMessageDialog(
									WindowFrame.this,
									"<html>Sorry, unable to save.  An unknown error occurred.",
									"Cannot Save File",
									JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		menus[0].add(menuItems[0][5]);

		//Save Session As
		menuItems[0][6] = new JMenuItem("Save Session As");
		menuItems[0][6].setMnemonic(KeyEvent.VK_A);
		menuItems[0][6].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				KeyEvent.CTRL_MASK + KeyEvent.SHIFT_MASK));
		menuItems[0][6].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//on this menu select, user should be able to select
				//a file to save to.

				//get a file explorer...
				JFileChooser fc = new JFileChooser();
				fc.addChoosableFileFilter(new EyeTrackFileFilter("ses",
						"Session File Type"));

				int returnVal = fc.showSaveDialog(WindowFrame.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					try {
						sessionHandler.writeDataToFile(file);
					} catch (Exception ex) {
						JOptionPane
								.showMessageDialog(
										WindowFrame.this,
										"<html>Sorry, unable to save.  An unknown error occurred.",
										"Cannot Save File",
										JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});
		menus[0].add(menuItems[0][6]);

		menus[0].addSeparator();
		//Import Menu
		menuItems[0][7] = new JMenu("Import");
		menuItems[0][7].setMnemonic(KeyEvent.VK_I);

		//the import menuItems...
		//video!
		menuItems[0][8] = new JMenuItem("Video");
		menuItems[0][8].setMnemonic(KeyEvent.VK_V);
		menuItems[0][8].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V,
				KeyEvent.CTRL_MASK));
		if (!hasQuicktime)
			menuItems[0][8].setEnabled(false);
		else
			menuItems[0][8].setEnabled(true);

		menuItems[0][8].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser fc = new JFileChooser(lastDirectory);
				int returnVal = fc.showDialog(WindowFrame.this, "Import");
				if (returnVal == JFileChooser.APPROVE_OPTION) {

					//give video to the session handler
					try {
						((PupilSessionHandler) sessionHandler).importVideo(fc
								.getSelectedFile());
						lastDirectory = fc.getCurrentDirectory();
					} catch (Exception e) {
						e.printStackTrace();
						//something went wrong, display error
						JOptionPane
								.showMessageDialog(
										WindowFrame.this,
										"<html>Sorry, unable to play this video.<br>"
												+ "Quicktime does not recognize the file format of <br>"
												+ fc.getSelectedFile()
														.getAbsolutePath(),
										"Cannot Import File",
										JOptionPane.ERROR_MESSAGE);
						return;
					}
					setEnableDisableMenus();
				}
			}
		});

		//task model!
		menuItems[0][9] = new JMenuItem("Task Model");
		menuItems[0][9].setMnemonic(KeyEvent.VK_T);
		menuItems[0][9].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M,
				KeyEvent.CTRL_MASK));
		menuItems[0][9].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//get a file explorer...
				JFileChooser fc = new JFileChooser(lastDirectory);
				fc.addChoosableFileFilter(new EyeTrackFileFilter("tsk",
						"Task Model"));

				int returnVal = fc.showDialog(WindowFrame.this, "Import");
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					lastDirectory = fc.getCurrentDirectory();

					//attempt to read give task model to session handler
					try {
						((PupilSessionHandler) sessionHandler)
								.importTaskModel(file);
					} catch (Exception e) {
						e.printStackTrace();
						JOptionPane
								.showMessageDialog(
										WindowFrame.this,
										"<html>Sorry, unable to parse <br>"
												+ file.getAbsolutePath()
												+ "<br>Check the syntax of this file for errors.",
										"Cannot Import File",
										JOptionPane.ERROR_MESSAGE);
						return;
					}

					//and the task model context menu menuItems...
					JMenuItem[] taskItems = ((PupilSessionHandler) sessionHandler)
							.getTaskMenuItems();
					if (taskItems != null && taskItems.length != 0) {
						WindowFrame.this.menuItems[1][6].removeAll();
						for (int i = 0; i < taskItems.length; i++)
							WindowFrame.this.menuItems[1][6].add(taskItems[i]);
					}

					//check if we need to reassess menues
					setEnableDisableMenus();
				}
			}
		});

		//pupil data!
		menuItems[0][10] = new JMenuItem("Pupil Data");
		menuItems[0][10].setMnemonic(KeyEvent.VK_P);
		menuItems[0][10].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
				KeyEvent.CTRL_MASK));
		menuItems[0][10].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//get a file explorer...
				JFileChooser fc = new JFileChooser(lastDirectory);
				fc.addChoosableFileFilter(new EyeTrackFileFilter("asc",
						"Pupil Data"));

				int returnVal = fc.showDialog(WindowFrame.this, "Import");
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					lastDirectory = fc.getCurrentDirectory();
					//read the pupil data

					try {
						((PupilSessionHandler) sessionHandler)
								.importPupilData(file);
					} catch (Exception e1) {
						JOptionPane
								.showMessageDialog(
										WindowFrame.this,
										"<html>Sorry, unable to parse <br>"
												+ file.getAbsolutePath()
												+ "<br>Check the syntax of this file for errors.",
										"Cannot Import File",
										JOptionPane.ERROR_MESSAGE);
						return;
					}

					//get the pupil context menu from the session handler
					JMenuItem[] pupilItems = ((PupilSessionHandler) sessionHandler)
							.getPupilTrialMenuItems();
					if (pupilItems != null && pupilItems.length != 0) {
						WindowFrame.this.menuItems[1][5].removeAll();
						for (int i = 0; i < pupilItems.length; i++)
							WindowFrame.this.menuItems[1][5].add(pupilItems[i]);

					}

					//check if we need to reassess menus
					setEnableDisableMenus();
				}
			}
		});

		//add menu menuItems to import menu
		menuItems[0][7].add(menuItems[0][8]);
		menuItems[0][7].add(menuItems[0][9]);
		menuItems[0][7].add(menuItems[0][10]);
		menus[0].add(menuItems[0][7]);

		menus[0].addSeparator();

		//Exit
		menuItems[0][11] = new JMenuItem("Exit");
		menuItems[0][11].setMnemonic(KeyEvent.VK_X);
		menuItems[0][11].setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_ESCAPE, 0));
		menuItems[0][11].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//clean up and exit
				cleanUpAndExit();
			}
		});
		menus[0].add(menuItems[0][11]);

		//View menu
		menus[1] = new JMenu("View");
		menus[1].setMnemonic(KeyEvent.VK_V);
		menuItems[1] = new JMenuItem[7];
		menuItems[1][0] = new JMenu("Local Frame");
		menuItems[1][0].setMnemonic(KeyEvent.VK_L);

		menuItems[1][1] = new JMenuItem("Zoom in");
		menuItems[1][1].setMnemonic(KeyEvent.VK_I);
		menuItems[1][1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_ADD,
				0));
		menuItems[1][1].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//zoom in by 10%
				zoomFocusBy(.10);
			}

		});

		menuItems[1][2] = new JMenuItem("Zoom out");
		menuItems[1][2].setMnemonic(KeyEvent.VK_O);
		menuItems[1][2].setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_SUBTRACT, 0));
		menuItems[1][2].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//zoom out by 10%
				zoomFocusBy(-.10);
			}
		});

		menuItems[1][3] = new JMenuItem("Zoom to Selection");
		menuItems[1][3].setMnemonic(KeyEvent.VK_S);
		menuItems[1][3].setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_MULTIPLY, 0));
		menuItems[1][3].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//get the min and max selection times...
				long minSelect = sessionHandler.getSelectedViewMinSelect();
				long maxSelect = sessionHandler.getSelectedViewMaxSelect();

				//set the focus if the selection times are valid
				if (minSelect != maxSelect)
					sessionHandler.setSelectedViewLocal(minSelect, maxSelect,
							null);
			}

		});

		menuItems[1][0].add(menuItems[1][1]);
		menuItems[1][0].add(menuItems[1][2]);
		menuItems[1][0].add(menuItems[1][3]);
		menus[1].add(menuItems[1][0]);

		//Global menu
		menuItems[1][4] = new JMenu("Global Frame");
		menuItems[1][4].setMnemonic(KeyEvent.VK_G);

		menuItems[1][5] = new JMenu("Zoom to Pupil Feature");
		menuItems[1][5].setMnemonic(KeyEvent.VK_P);
		menuItems[1][5].add(new JMenuItem("All Trials"));

		menuItems[1][6] = new JMenu("Zoom to Task Feature");
		menuItems[1][6].setMnemonic(KeyEvent.VK_T);
		menuItems[1][6].add(new JMenuItem("Entire Task Model"));

		menuItems[1][4].add(menuItems[1][5]);
		menuItems[1][4].add(menuItems[1][6]);
		menus[1].add(menuItems[1][4]);

		//Select menu
		menus[2] = new JMenu("Select");
		menus[2].setMnemonic(KeyEvent.VK_S);
		menuItems[2] = new JMenuItem[1];
		menuItems[2][0] = new JMenuItem("Select All in Local View");
		menuItems[2][0].setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_DIVIDE, 0));
		menuItems[2][0].setMnemonic(KeyEvent.VK_A);
		menuItems[2][0].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//get the focus window
				long minFocus = sessionHandler.getSelectedViewMinLocal();
				long maxFocus = sessionHandler.getSelectedViewMaxLocal();

				//set the selection window (assuming valid window)
				if (minFocus != maxFocus)
					sessionHandler.setSelectedViewSelection(minFocus, maxFocus,
							null);
			}
		});
		menus[2].add(menuItems[2][0]);

		//Tools menu
		menus[3] = new JMenu("Tools");
		menus[3].setMnemonic(KeyEvent.VK_T);
		menuItems[3] = new JMenuItem[5];


		//Pupil Mask Settings
		menuItems[3][0] = new JMenuItem("Pupil Filter");
		menuItems[3][0].setMnemonic(KeyEvent.VK_S);
		menuItems[3][0].setEnabled(true);
		menuItems[3][0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				KeyEvent.SHIFT_MASK + KeyEvent.CTRL_MASK));
		menuItems[3][0].addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				//show pupil mask dialog
				((PupilSessionHandler) sessionHandler).dispPupilFilterSettings();
				WindowFrame.this.repaint();
			}

		});
		menus[3].add(menuItems[3][0]);




		//Pupil Mask Settings
		menuItems[3][1] = new JMenuItem("Pupil Mask");
		menuItems[3][1].setMnemonic(KeyEvent.VK_M);
		menuItems[3][1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M,
				KeyEvent.SHIFT_MASK + KeyEvent.CTRL_MASK));
		menuItems[3][1].addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				//show pupil mask dialog
				((PupilSessionHandler) sessionHandler).dispPupilMaskSettings();
			}

		});
		menus[3].add(menuItems[3][1]);

		//Statistics
		menuItems[3][2] = new JMenuItem("Statistics");
		menuItems[3][2].setMnemonic(KeyEvent.VK_S);
		menuItems[3][2]
				.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, 0));
		menuItems[3][2].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//toggle statistics being shown
				if (((PupilSessionHandler) sessionHandler).isShowStats()) {
					((PupilSessionHandler) sessionHandler).setShowStats(false);
					menuItems[3][2].setText("Statistics");
				} else {
					((PupilSessionHandler) sessionHandler).setShowStats(true);
					menuItems[3][2].setText("Hide Statistics");
				}
			}
		});
		menus[3].add(menuItems[3][2]);

		//Notes
		menuItems[3][3] = new JMenuItem("Notes");
		menuItems[3][3].setMnemonic(KeyEvent.VK_N);
		menuItems[3][3]
				.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, 0));
		menuItems[3][3].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//toggle notes being shown
				if (((PupilSessionHandler) sessionHandler).isShowNotes()) {
					((PupilSessionHandler) sessionHandler).setShowNotes(false);
					menuItems[3][3].setText("Notes");
				} else {
					((PupilSessionHandler) sessionHandler).setShowNotes(true);
					menuItems[3][3].setText("Hide Notes");
				}

			}
		});
		menus[3].add(menuItems[3][3]);

		//Video
		menuItems[3][4] = new JMenuItem("Video");
		menuItems[3][4].setMnemonic(KeyEvent.VK_V);
		menuItems[3][4]
				.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, 0));
		menuItems[3][4].addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//toggle video being shown
				if (((PupilSessionHandler) sessionHandler).isShowVideo()) {
					((PupilSessionHandler) sessionHandler).setShowVideo(false);
					menuItems[3][4].setText("Video");
				} else {
					((PupilSessionHandler) sessionHandler).setShowVideo(true);
					menuItems[3][4].setText("Hide Video");
				}
			}
		});
		menus[3].add(menuItems[3][4]);

		//give the windows menu to the session Handler...
		((PupilSessionHandler) sessionHandler).setWindowItems(menuItems[3]);

		for (int i = 0; i < menus.length; i++)
			mb.add(menus[i]);

		//add menu bar to window
		setJMenuBar(mb);
	}

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		WindowFrame et = new WindowFrame("Taprav");
		et.setVisible(true);
	}

	/**
	 * method cleans up whatever it needs to and exits the program
	 */
	public void cleanUpAndExit() {
		//close quicktime session. get out.
		try {
			MYQTSession.close();
		} catch (NoClassDefFoundError e) {
			//don't really care...
		} catch (Exception e) {
			//don't really care...
		}
		System.exit(0);
	}

	/**
	 * method zooms the focus of the selected view by some value.
	 * (value=0;nochange, value>0;zoom in, value <0; zoom out)
	 * 
	 * @param value
	 */
	private void zoomFocusBy(double value) {
		double currentMin = sessionHandler.getSelectedViewMinLocal();
		double currentMax = sessionHandler.getSelectedViewMaxLocal();

		if (currentMax != currentMin) {
			//the midpoint of min and max
			double avg = (currentMax + currentMin) / 2;

			//get the new min and max
			long newMin = (long) (avg - (.5 - value / 2)
					* (currentMax - currentMin));
			long newMax = (long) (avg + (.5 - value / 2)
					* (currentMax - currentMin));

			sessionHandler.setSelectedViewLocal(newMin, newMax, null);
		}
	}

}