/*
 * Created on Sep 16, 2004
 * PupilVisWindow.java
 * 
 * Project: eyetrack_cvs
 */
package pupil;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;
import java.util.Vector;

import view.VisWindow;

/**
 * @author chris
 *  
 */
public class PupilVisWindow extends VisWindow {

	/** the buffered image for this view* */
	private BufferedImage viewBuffer = null;

	/** the current min time for the buffer */
	private long minBufTime = 0;

	/** the current max time for the buffer */
	private long maxBufTime = 0;

	/** the current min time for the view* */
	private long minViewTime = 0;

	/** the current max time for the view* */
	private long maxViewTime = 0;

	/** flag for drawing baseline */
	private boolean willDrawBaseLine = true;

	/** the maximum value of PCPS */
	private float maxPCPS;

	/** the min value of PCPS* */
	private float minPCPS;

	/** the pupil data to draw */
	private Vector data;

	/** the baseline pupil size */
	private float baseLine;

	/** decimal formate for drawing labels */
	private DecimalFormat df = new DecimalFormat("##");

	/** the masking threshold* */
	private float maskThresh;

	/** flag for forcing a redraw* */
	private boolean redraw = false;

	/**
	 * flag indicating whether the window will be rescaled during masking
	 */
	private boolean willReScaleOnMask = true;

	/** if not scaling on mask, save the "old" mask */
	private float oldMask;

	protected Rectangle drawRegion;

	/**
	 * default construcot
	 * 
	 * @param drawRegion -
	 *            the drawing region
	 */
	public PupilVisWindow(Rectangle drawRegion) {
		this.drawRegion = drawRegion;
	}

	/**
	 * method draws the labels for this view
	 * 
	 * @param g2
	 * @param f
	 *            the font to use
	 */
	public void drawLabels(Graphics2D g2, Font f) {
		if (data != null) {
			g2.setColor(Color.GRAY);
			FontRenderContext frc = g2.getFontRenderContext();

			int y;
			for (float i = .10f; i < maxPCPS; i += .10) {
				if (willReScaleOnMask)
					y = (int) (drawRegion.getMaxY() - ((i - maskThresh) / (maxPCPS - maskThresh))
							* (drawRegion.getMaxY() - drawRegion.getMinY()));
				else
					y = (int) (drawRegion.getMaxY() - ((i - oldMask) / (maxPCPS - oldMask))
							* (drawRegion.getMaxY() - drawRegion.getMinY()));

				g2.drawLine(drawRegion.x, y, drawRegion.x - 2, y);
				g2.drawLine((int) drawRegion.getMaxX(), y, (int) drawRegion
						.getMaxX() + 2, y);
			}

			for (float i = -.10f; i > maskThresh; i -= .10) {
				if (willReScaleOnMask)
					y = (int) (drawRegion.getMaxY() - ((i - maskThresh) / (maxPCPS - maskThresh))
							* (drawRegion.getMaxY() - drawRegion.getMinY()));
				else
					y = (int) (drawRegion.getMaxY() - ((i - oldMask) / (maxPCPS - oldMask))
							* (drawRegion.getMaxY() - drawRegion.getMinY()));
				g2.drawLine(drawRegion.x, y, drawRegion.x - 2, y);
				g2.drawLine((int) drawRegion.getMaxX(), y, (int) drawRegion
						.getMaxX() + 2, y);
			}

			String s = df.format(100 * maxPCPS) + "%";
			y = drawRegion.y;
			TextLayout tl = new TextLayout(s, f, frc);
			tl.draw(g2, (float) (drawRegion.getMinX()
					- tl.getBounds().getWidth() - 5), (float) (y + tl
					.getBounds().getHeight()));
			tl.draw(g2, (float) (drawRegion.getMaxX() + 6), (float) (y + tl
					.getBounds().getHeight()));
			g2.drawLine(drawRegion.x, y, drawRegion.x - 4, y);
			g2.drawLine((int) drawRegion.getMaxX(), y, (int) drawRegion
					.getMaxX() + 4, y);

			if (willReScaleOnMask)
				s = df.format(100 * maskThresh) + "%";
			else
				s = df.format(100 * oldMask) + "%";
			y = (int) drawRegion.getMaxY();
			tl = new TextLayout(s, f, frc);
			tl.draw(g2, (float) (drawRegion.getMinX()
					- tl.getBounds().getWidth() - 5), y);
			tl.draw(g2, (float) (drawRegion.getMaxX() + 6), y);
			g2.drawLine(drawRegion.x, y, drawRegion.x - 4, y);
			g2.drawLine((int) drawRegion.getMaxX(), y, (int) drawRegion
					.getMaxX() + 4, y);

			if (willReScaleOnMask) {
				if (0 < maskThresh)
					return;
				y = (int) (drawRegion.getMaxY() + (maskThresh / (maxPCPS - maskThresh))
						* drawRegion.height);
			} else {
				if (0 < oldMask)
					return;
				y = (int) (drawRegion.getMaxY() + (oldMask / (maxPCPS - oldMask))
						* drawRegion.height);
			}

			s = df.format(0) + "%";
			tl = new TextLayout(s, f, frc);

			g2.drawLine(drawRegion.x, y, drawRegion.x - 4, y);
			g2.drawLine((int) drawRegion.getMaxX(), y, (int) drawRegion
					.getMaxX() + 4, y);
			tl.draw(g2, (float) (drawRegion.getMinX()
					- tl.getBounds().getWidth() - 5), (float) (y + tl
					.getBounds().getHeight() / 2));
			tl.draw(g2, (float) (drawRegion.getMaxX() + 6), (float) (y + tl
					.getBounds().getHeight() / 2));
		}
	}

	/**
	 * method draws the data for this view
	 * 
	 * @param g2
	 */
	public void drawWindow(Graphics2D g2) {
		//only do something if we have data
		if (data == null)
			return;

		//initialize the buffered region if we have to
		if (viewBuffer == null || viewBuffer.getWidth() != drawRegion.width
				|| viewBuffer.getHeight() != drawRegion.height) {

			//initialize the focusBuffer...
			viewBuffer = new BufferedImage(drawRegion.width, drawRegion.height,
					BufferedImage.TYPE_4BYTE_ABGR);

			//update the times...
			minBufTime = minViewTime;
			maxBufTime = maxViewTime;

			Graphics2D tmpG = viewBuffer.createGraphics();
			Rectangle tmp = new Rectangle(0, 0, viewBuffer.getWidth(),
					viewBuffer.getHeight());
			tmpG.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR,
					0.0f));
			tmpG.fill(tmp);
			tmpG.setComposite(AlphaComposite.SrcOver);
			tmpG.setColor(Color.BLACK);
			drawPupilCurve(tmpG);
		}

		//check if the buffered image has expired or if we have to redraw
		if (minBufTime != minViewTime || maxBufTime != maxViewTime || redraw) {
			redraw = false;
			//reset the times...
			minBufTime = minViewTime;
			maxBufTime = maxViewTime;

			Graphics2D tmpG = viewBuffer.createGraphics();
			Rectangle tmp = new Rectangle(0, 0, viewBuffer.getWidth(),
					viewBuffer.getHeight());
			tmpG.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR,
					0.0f));
			tmpG.fill(tmp);
			tmpG.setComposite(AlphaComposite.SrcOver);
			tmpG.setColor(Color.BLACK);
			drawPupilCurve(tmpG);
		}

		if (minViewTime != maxViewTime) {

			//draw baseline
			if (willDrawBaseLine)
				drawBaseLine(g2);

			//focus buffer
			g2.drawImage(viewBuffer, (int) drawRegion.getMinX(),
					(int) drawRegion.getMinY(), null);
		}

	}

	/**
	 * @param g2
	 */
	private void drawBaseLine(Graphics2D g2) {

		int y;
		if (willReScaleOnMask) {
			if (0 < maskThresh)
				return;

			g2.setColor(Color.RED);
			g2.setStroke(new BasicStroke(2.0f));
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					.15f));
			y = (int) (drawRegion.getMaxY() + (maskThresh / (maxPCPS - maskThresh))
					* drawRegion.height);
		} else {
			if (0 < oldMask)
				return;

			g2.setColor(Color.RED);
			g2.setStroke(new BasicStroke(1.0f));
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					.15f));
			y = (int) (drawRegion.getMaxY() + (oldMask / (maxPCPS - oldMask))
					* drawRegion.height);
		}

		g2.drawLine(drawRegion.x, y, drawRegion.x + drawRegion.width, y);
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER));
	}

	/**
	 * method draws the pupil curve. this should be called on the image buffer
	 * 
	 * @param g2
	 */
	private void drawPupilCurve(Graphics2D g2) {
		GeneralPath pupilLine = new GeneralPath(GeneralPath.WIND_EVEN_ODD, data
				.size());

		g2.setColor(Color.black);
		g2.setStroke(new BasicStroke(1));

		//we need a rectangle located at 0,0
		Rectangle clipRect = new Rectangle(drawRegion);
		clipRect.setLocation(0, 0);

		//the last pupil data packet before the time window that is not below
		// the threshhold
		PupilDataPacket lastValidPreStamp = null;

		double x, y;

		//a counter that indicates the 2nd thing to draw.
		int i = 0;
		for (; i < data.size(); i++) {
			PupilDataPacket PD = (PupilDataPacket) data.get(i);

			//don't draw packets we don't need to...

			//forget about this case...
			if (PD.getTimeStamp() > maxViewTime)
				return;

			//before our view...
			if (PD.getTimeStamp() < minViewTime) {
				//is it valid?
				if ((PD.getPupilSize() - baseLine) / baseLine < maskThresh)
					continue;

				lastValidPreStamp = PD;
				continue;
			}

			//is there a valid prestamp? if so, start the line
			if (lastValidPreStamp != null) {
				pupilLine.moveTo((float) getXCoord(lastValidPreStamp
						.getTimeStamp(), clipRect), (float) getYCoord(
						lastValidPreStamp.getPupilSize(), clipRect));

				if ((PD.getPupilSize() - baseLine) / baseLine >= maskThresh)
					pupilLine.lineTo((float) getXCoord(PD.getTimeStamp(),
							clipRect), (float) getYCoord(PD.getPupilSize(),
							clipRect));
				break;
			}

			//can we start the line with packet?
			if ((PD.getPupilSize() - baseLine) / baseLine >= maskThresh) {
				pupilLine.moveTo(
						(float) getXCoord(PD.getTimeStamp(), clipRect),
						(float) getYCoord(PD.getPupilSize(), clipRect));
				break;
			}

			//otherwise, keep going...
		}

		//draw the rest...
		for (i++; i < data.size(); i++) {
			PupilDataPacket PD = (PupilDataPacket) data.get(i);
			if ((PD.getPupilSize() - baseLine) / baseLine < maskThresh)
				continue;
			x = getXCoord(PD.getTimeStamp(), clipRect);
			y = getYCoord(PD.getPupilSize(), clipRect);

			pupilLine.lineTo((float) x, (float) y);

			//don't draw anymore packets we don't need to...
			if (PD.getTimeStamp() > maxViewTime)
				break;
		}

		g2.draw(pupilLine);
	}

	/**
	 * given some time, method returns the x coords of the time
	 * 
	 * @param time -
	 *            the time in question
	 */
	public int getXCoord(double time) {
		return getXCoord(time, drawRegion);
	}

	/**
	 * given some time and rect, method returns the x coord. should be used to
	 * draw ono the buffered region
	 * 
	 * @param time
	 * @param r
	 * @return
	 */
	private int getXCoord(double time, Rectangle r) {
		if (minViewTime == maxViewTime)
			return -1;
		return (int) (r.getMinX() + ((r.getWidth() - 1) * (time - minViewTime) / (maxViewTime - minViewTime)));
	}

	/**
	 * given some pupil size and rectangle, method returns the y value at that
	 * point
	 * 
	 * @param pupilSize
	 * @param r
	 * @return
	 */
	private double getYCoord(float pupilSize, Rectangle r) {
		if (willReScaleOnMask)
			return (int) (r.getMaxY() - (((pupilSize - baseLine) / baseLine - maskThresh) / (maxPCPS - maskThresh))
					* (r.getMaxY() - r.getMinY()));
		else
			return (int) (r.getMaxY() - (((pupilSize - baseLine) / baseLine - oldMask) / (maxPCPS - oldMask))
					* (r.getMaxY() - r.getMinY()));
	}

	/**
	 * @param data
	 *            The data to set.
	 */
	public void setData(Vector data) {
		this.data = data;
	}

	/**
	 * @param baseLine
	 *            The baseLine to set.
	 */
	public void setBaseLine(float baseLine) {
		this.baseLine = baseLine;
	}

	/**
	 * method sets the boundary pcps
	 * 
	 * @param minPCPS
	 * @param maxPCPS
	 */
	public void setExtremePCPS(float minPCPS, float maxPCPS) {
		this.minPCPS = minPCPS;
		this.maxPCPS = maxPCPS;
	}

	/**
	 * method sets the current view time boundaries
	 * 
	 * @param minViewTime
	 * @param maxViewTime
	 */
	public void setViewTimes(long minViewTime, long maxViewTime) {
		this.minViewTime = minViewTime;
		this.maxViewTime = maxViewTime;
	}

	/**
	 * given some x coordinate in the view, method returns the time at that
	 * point
	 * 
	 * @param xPos
	 * @return the time
	 */
	public double getTime(int xPos) {
		if (drawRegion.width == 0)
			return -1;
		return minViewTime + (maxViewTime - minViewTime)
				* (xPos - drawRegion.getMinX()) / (drawRegion.width);
	}

	/**
	 * @return Returns the willDrawBaseLine.
	 */
	public boolean isWillDrawBaseLine() {
		return willDrawBaseLine;
	}

	/**
	 * @param willDrawBaseLine
	 *            The willDrawBaseLine to set.
	 */
	public void setWillDrawBaseLine(boolean willDrawBaseLine) {
		this.willDrawBaseLine = willDrawBaseLine;
	}

	/**
	 * method sets the mask threshold for this view
	 * 
	 * @param maskThresh
	 */
	public void setMaskThreshold(float maskThresh, boolean willReScaleOnMask) {
		this.maskThresh = maskThresh;
		this.willReScaleOnMask = willReScaleOnMask;
		if (willReScaleOnMask)
			oldMask = this.maskThresh;
	}

	/**
	 * method forces view to redraw it's image buffer the next time it is
	 * repainted.
	 */
	public void forceRedraw() {
		redraw = true;
	}

	public String getToolTip(java.awt.event.MouseEvent e) {
		double mouseTime = getTime(e.getX());

		//search pupil data for this time
		float PCPS = 0;
		int minIndex = 0, maxIndex = data.size();
		int midIndex = 0;
		PupilDataPacket midPacket = null;

		while (true) {
			midIndex = (minIndex + maxIndex) / 2;
			midPacket = (PupilDataPacket) data.get(midIndex);

			//match?
			if (midPacket.getTimeStamp() == mouseTime) {
				//is it valid?
				PCPS = (midPacket.getPupilSize() - baseLine) / baseLine;
				if (PCPS < maskThresh)
					break;
				else
					return "<html> " + PCPS + " %<br>" + df.format(mouseTime);
			}

			//time is below? recurse!
			if (midPacket.getTimeStamp() > mouseTime) {
				maxIndex = midIndex;
				continue;
			}

			//match inbetween?
			PupilDataPacket nextPacket = (PupilDataPacket) data
					.get(midIndex + 1);
			if (midPacket.getTimeStamp() < mouseTime
					&& nextPacket.getTimeStamp() > mouseTime)
				break;

			//otherwise, recurse...
			minIndex = midIndex;
		}

		//if the time is valid...
		if (midPacket != null) {
			//find the prev and next valid packets for this point
			PupilDataPacket prev = null, next = null;

			//min and max index are used as indices to search for prev
			//and next
			//did we get an exact (invalid) match?
			if (midPacket.getTimeStamp() == mouseTime) {
				minIndex = midIndex - 1;
				maxIndex = midIndex + 1;
			} else {
				//not an exact match... interpolate from neighbors
				minIndex = midIndex;
				maxIndex = midIndex + 1;
			}
			//no point in searching if no where to search...
			if (minIndex < 0 || maxIndex > data.size() - 1)
				return null;

			//find prev..
			for (; minIndex >= 0; minIndex--) {
				if ((((PupilDataPacket) data.get(minIndex)).getPupilSize() - baseLine)
						/ baseLine < maskThresh)
					continue;
				else {
					prev = (PupilDataPacket) data.get(minIndex);
					break;
				}
			}

			//no prev? who cares...
			if (prev == null)
				return null;

			//find next..
			for (; maxIndex < data.size(); maxIndex++) {
				if ((((PupilDataPacket) data.get(maxIndex)).getPupilSize() - baseLine)
						/ baseLine < maskThresh)
					continue;
				else {
					next = (PupilDataPacket) data.get(maxIndex);
					break;
				}
			}

			//no next? who cares...
			if (next == null)
				return null;

			double deltat = next.getTimeStamp() - prev.getTimeStamp();

			PCPS = 100
					* ((float) ((1 - (mouseTime - prev.getTimeStamp()) / deltat)
							* prev.getPupilSize() + ((mouseTime - prev
							.getTimeStamp()) / deltat)
							* next.getPupilSize()) - baseLine) / baseLine;

			DecimalFormat df = new DecimalFormat("#.##");
			return "<html> " + PCPS + " %<br>" + df.format(mouseTime);
		}
		//just in case...
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getMinViewTime()
	 */
	public long getMinViewTime() {
		return minViewTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getMaxViewTime()
	 */
	public long getMaxViewTime() {
		return maxViewTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setDrawRegion(java.awt.Rectangle)
	 */
	public void setDrawRegion(Rectangle drawRegion) {
		this.drawRegion = drawRegion;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getDrawRegion()
	 */
	public Rectangle getDrawRegion() {
		return this.drawRegion;
	}
}