/*
 * Created on Oct 31, 2004
 * SessionHandler.java
 * 
 * Project: eyetrack_cvs
 */
package session;

import java.io.File;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JPanel;
import view.View;

/**
 * @author chris
 *  
 */
public abstract class SessionHandler extends JPanel {

	/** the parent JFrame* */
	private JFrame owner;

	/** The file this Session is saved to */
	private File sessionFile;

	public SessionHandler(JFrame owner) {
		this.owner = owner;
	}

	public abstract void setSelectedViewSelection(long minTime, long maxTime,
			View theCallingView);

	public abstract void previewSelectedViewSelection(long minTime,
			long maxTime, View theCallingView);

	public abstract void setSelectedViewLocal(long minTime, long maxTime,
			View theCallingView);

	public abstract void previewSelectedViewLocal(long minTime, long maxTime,
			View theCallingView);

	public abstract void setSelectedViewGlobal(long minTime, long maxTime,
			View theCallingView);

	public abstract void previewSelectedViewGlobal(long minTime, long maxTime,
			View theCallingView);

	/**
	 * returns the current number of views
	 * 
	 * @return
	 */
	public abstract int getNumberOfViews();

	/**
	 * method adds a view to this session
	 *  
	 */
	public abstract void addView();

	/**
	 * method purges session of state
	 */
	public abstract void purge();

	public abstract void readDataFromFile(File f) throws IOException,
			ClassNotFoundException;

	public abstract void removeView();

	public abstract String getSelectedViewName();

	public abstract void setSelectedViewName(String name);

	public abstract void writeDataToFile() throws Exception;

	public abstract void writeDataToFile(File f) throws Exception;

	/**
	 * @return
	 */
	public abstract long getSelectedViewMinSelect();

	/**
	 * @return
	 */
	public abstract long getSelectedViewMaxSelect();

	/**
	 * @return
	 */
	public abstract long getSelectedViewMinLocal();

	/**
	 * @return
	 */
	public abstract long getSelectedViewMaxLocal();

	/**
	 * @return Returns the owner.
	 */
	public JFrame getOwner() {
		return owner;
	}

	/**
	 * @param owner
	 *            The owner to set.
	 */
	public void setOwner(JFrame owner) {
		this.owner = owner;
	}

	/**
	 * @return Returns the sessionFile.
	 */
	public File getSessionFile() {
		return sessionFile;
	}

	/**
	 * @param sessionFile
	 *            The sessionFile to set.
	 */
	public void setSessionFile(File sessionFile) {
		this.sessionFile = sessionFile;
	}
}