/*
 * Created on Sep 3, 2004
 * BaselineDialog.java
 * 
 * Project: eyetrack_cvs
 */

package pupil;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

/**
 * @author chris dialog box for getting the baseline pupil size or the time
 *         bounds for where the baseline was calculated.
 */
public class BaselineDialog extends JDialog {
	/** text field for min time */
	private JFormattedTextField minTime;

	/** text field for max time */
	private JFormattedTextField maxTime;

	/** text field for the baseline value */
	private JFormattedTextField baseLineField;

	/** OK button */
	private JButton okBu;

	/** CANCEL button */
	private JButton cancelBu;

	/** min time set by the user */
	private long min = -1;

	/** max time set by the user */
	private long max = -1;

	/** baseline set by the user */
	private float baseLine = -1;

	/** the absolute smallest time value possible */
	private long absoluteMin;

	/** the absolute largest time value possible */
	private long absoluteMax;

	/** the button group for the radio buttons */
	private ButtonGroup group;

	/** the radio button for entering time */
	private JRadioButton timeButton;

	/** the radio button for entering a baseline value */
	private JRadioButton baseLineButton;

	/** misc label */
	private JLabel minTimeLabel, maxTimeLabel;

	/**
	 * default constructor
	 * 
	 * @param owner -
	 *            the parent JFrame
	 * @param absoluteMinTime -
	 *            the absolute minimum valid time
	 * @param absoluteMaxTime -
	 *            the absolute maximum valid time
	 */
	public BaselineDialog(JFrame owner, long absoluteMinTime,
			long absoluteMaxTime) {
		super(owner, "Set Baseline");
		this.absoluteMin = absoluteMinTime;
		this.absoluteMax = absoluteMaxTime;

		//initialize the radio buttons
		baseLineButton = new JRadioButton("Baseline Value");
		baseLineButton.setMnemonic(KeyEvent.VK_B);
		baseLineButton.setSelected(true);
		timeButton = new JRadioButton("Boundary Times of Baseline Calculation");
		timeButton.setMnemonic(KeyEvent.VK_T);
		timeButton.setSelected(false);
		group = new ButtonGroup();
		group.add(baseLineButton);
		group.add(timeButton);
		baseLineButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				baseLineField.setEnabled(true);
				minTime.setEnabled(false);
				maxTime.setEnabled(false);
				minTimeLabel.setEnabled(false);
				maxTimeLabel.setEnabled(false);
			}
		});
		timeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				baseLineField.setEnabled(false);
				minTime.setEnabled(true);
				maxTime.setEnabled(true);
				minTimeLabel.setEnabled(true);
				maxTimeLabel.setEnabled(true);
			}
		});

		//initialize the other buttons
		okBu = new JButton("OK");
		okBu.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {

				//which option did the user select?
				if (timeButton.isSelected()) {
					//verify that the entered text is valid.
					long tmpMin = ((Number) minTime.getValue()).longValue();
					long tmpMax = ((Number) maxTime.getValue()).longValue();

					//does everything check out?

					//check if user entered min is greater than user entered
					// max
					if (tmpMin > tmpMax) {
						//present user with error message
						JOptionPane
								.showMessageDialog(
										BaselineDialog.this,
										"<html>The entered times are not valid.<br><br>"
												+ "The Maximum time of the baseline interval must be <br>"
												+ "greater than the Minimum time.",
										"Error", JOptionPane.ERROR_MESSAGE);
					} else if (tmpMin < BaselineDialog.this.absoluteMin
							|| tmpMax > BaselineDialog.this.absoluteMax) {
						//present user with error message
						JOptionPane
								.showMessageDialog(
										BaselineDialog.this,
										"<html>The entered times are not valid.<br><br>"
												+ "The entered times must fall in the range of the <br>"
												+ "collected pupil data; "
												+ BaselineDialog.this.absoluteMin
												+ "-"
												+ BaselineDialog.this.absoluteMax
												+ ".", "Error",
										JOptionPane.ERROR_MESSAGE);
					} else {
						min = tmpMin;
						max = tmpMax;

						setVisible(false);
					}
				} else {
					//verify that the entered text is valid.
					float tmpbl = ((Number) baseLineField.getValue())
							.floatValue();

					//does this check out?
					if (tmpbl <= 0) {
						//present user with error message
						JOptionPane
								.showMessageDialog(
										BaselineDialog.this,
										"<html>The entered baseline not valid.<br><br>"
												+ "The baseline must be greater than 0.",
										"Error", JOptionPane.ERROR_MESSAGE);
					} else {
						baseLine = tmpbl;

						setVisible(false);
					}
				}
			}

		});
		cancelBu = new JButton("Cancel");
		cancelBu.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				BaselineDialog.this.setVisible(false);
			}

		});

		//initialize the text fields
		NumberFormat nf = NumberFormat.getNumberInstance();
		nf.setGroupingUsed(false);
		baseLineField = new JFormattedTextField(nf);
		baseLineField.setValue(new Float(0.0));
		baseLineField.setMaximumSize(new Dimension(baseLineField
				.getPreferredSize().width * 6,
				baseLineField.getPreferredSize().height));

		minTime = new JFormattedTextField(nf);
		minTime.setValue(new Double(absoluteMinTime));
		minTime.setEnabled(false);
		maxTime = new JFormattedTextField(nf);
		maxTime.setValue(new Double(absoluteMaxTime));
		maxTime.setEnabled(false);

		// just a friendly label.
		JPanel labelPanel = new JPanel();
		labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.LINE_AXIS));
		JLabel alabel = new JLabel(
				"<html>Enter the Baseline pupil size, or enter the Min and Max boundary times<br> of the interval where the baseline was measured.");
		labelPanel.add(alabel);
		labelPanel.add(Box.createHorizontalGlue());
		labelPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		//the baseline value panel
		JPanel baseLinePanel = new JPanel();
		baseLinePanel.setLayout(new BoxLayout(baseLinePanel,
				BoxLayout.LINE_AXIS));
		baseLinePanel.add(baseLineButton);
		baseLinePanel.add(Box.createRigidArea(new Dimension(10, 0)));
		baseLinePanel.add(baseLineField);

		//the time boundary panel
		JPanel timePanel = new JPanel();
		timePanel.setLayout(new BoxLayout(timePanel, BoxLayout.LINE_AXIS));
		timePanel.add(timeButton);

		//the input panel holds the time text fields
		JPanel timeInputPanel = new JPanel();
		timeInputPanel.setLayout(new BoxLayout(timeInputPanel,
				BoxLayout.LINE_AXIS));
		JPanel textPanel = new JPanel();
		textPanel.setLayout(new GridLayout(2, 2));
		minTimeLabel = new JLabel("Min Time: ");
		minTimeLabel.setEnabled(false);
		textPanel.add(minTimeLabel);
		textPanel.add(minTime);
		maxTimeLabel = new JLabel("Max Time: ");
		maxTimeLabel.setEnabled(false);
		textPanel.add(maxTimeLabel);
		textPanel.add(maxTime);
		timeInputPanel.add(Box.createHorizontalGlue());
		timeInputPanel.add(textPanel);
		timeInputPanel.add(Box.createHorizontalGlue());

		//the button panel holds the ok and cancel buttons
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
		buttonPanel.add(okBu);
		buttonPanel.add(cancelBu);

		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.PAGE_AXIS));
		this.getContentPane().add(labelPanel);
		this.getContentPane().add(Box.createRigidArea(new Dimension(0, 10)));
		this.getContentPane().add(baseLinePanel);
		this.getContentPane().add(Box.createRigidArea(new Dimension(0, 10)));
		this.getContentPane().add(timePanel);
		this.getContentPane().add(timeInputPanel);
		this.getContentPane().add(buttonPanel);

		this.pack();
	}

	/**
	 * method should be called after dialog is shown to be visible.
	 * 
	 * @return true if data was entered, false if cancel pressed
	 */
	public boolean isDataEntered() {
		if (min == max && min == -1 && baseLine == -1)
			return false;
		else
			return true;
	}

	/**
	 * @return the min time value entered by the user
	 */
	public long getMin() {
		return min;
	}

	/**
	 * @return the max time value entered by the user
	 */
	public long getMax() {
		return max;
	}

	/**
	 * 
	 * @return true if baseline entered. false otherwise
	 */
	public boolean haveBaseLine() {
		if (baseLine != -1)
			return true;
		else
			return false;
	}

	/**
	 * @return the user entered baseline value
	 */
	public float getBaseLine() {
		return baseLine;
	}
}