/*
 * Created on Oct 16, 2004
 * PupilMaskDialog.java
 * 
 * Project: eyetrack_cvs
 */
package pupil;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.NumberFormat;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.NumberFormatter;

/**
 * @author chris
 *  
 */
public class PupilMaskDialog extends JDialog {
	/** the slider used to get the threshold */
	private JSlider maskThresholdSlider;

	/** dataEntered is true if data was entered- ie, ok button pushed */
	private boolean dataEntered = false;

	/** the session handler* */
	private PupilSessionHandler handler;

	/** the old mask setting* */
	private float oldMask;

	/** the text field going along with the slider */
	private JFormattedTextField textField;

	/**
	 * default constructor
	 * 
	 * @param owner -
	 *            the parent frame
	 * @param minPCPS -
	 *            the min pcps
	 * @param maxPCPS -
	 *            the max pcps
	 * @param currentMask -
	 *            the current mask
	 */
	public PupilMaskDialog(JFrame owner, PupilSessionHandler handler,
			float minPCPS, float maxPCPS, float currentMask) {
		super(owner, "Set Pupil Mask");
		this.handler = handler;
		oldMask = currentMask;

		//initialize the slider
		maskThresholdSlider = new JSlider(JSlider.HORIZONTAL,
				(int) (minPCPS * 100), (int) (maxPCPS * 100),
				(int) (currentMask * 100));
		maskThresholdSlider.setMajorTickSpacing(10);
		maskThresholdSlider.setPaintTicks(true);

		//create the label table
		Hashtable labelTable = new Hashtable();
		labelTable.put(new Integer((int) (minPCPS * 100)), new JLabel(
				(int) (minPCPS * 100) + "%"));
		labelTable.put(new Integer(0), new JLabel("0%"));
		labelTable.put(new Integer((int) (maxPCPS * 100)), new JLabel(
				(int) (maxPCPS * 100) + "%"));
		maskThresholdSlider.setLabelTable(labelTable);
		maskThresholdSlider.setPaintLabels(true);
		maskThresholdSlider.setMaximumSize(maskThresholdSlider
				.getPreferredSize());
		maskThresholdSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				if (maskThresholdSlider.getValueIsAdjusting()) {
					textField.setValue(new Integer(maskThresholdSlider
							.getValue()));
				} else {
					textField.setText(String.valueOf(maskThresholdSlider
							.getValue()));
				}
				PupilMaskDialog.this.handler
						.previewMaskSettings((float) maskThresholdSlider
								.getValue() / 100);
			}

		});

		//set up the text field
		NumberFormat numberFormat = NumberFormat.getIntegerInstance();
		NumberFormatter formatter = new NumberFormatter(numberFormat);
		formatter.setMinimum(new Integer((int) (100 * minPCPS)));
		formatter.setMaximum(new Integer((int) (100 * maxPCPS)));
		textField = new JFormattedTextField(formatter);
		textField.setValue(new Integer((int) (100 * currentMask)));
		textField.setColumns(3);
		textField.setMaximumSize(textField.getPreferredSize());
		textField.addPropertyChangeListener(new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent e) {
				Number value = (Number) textField.getValue();
				if (maskThresholdSlider != null) {
					maskThresholdSlider.setValue(value.intValue());
				}

			}

		});

		//add components to input panel
		JPanel inputPanel = new JPanel();
		inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.PAGE_AXIS));
		JPanel tmpPanel = new JPanel();
		tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.LINE_AXIS));
		tmpPanel.add(new JLabel("PCPS Masking Threshold"));
		tmpPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		tmpPanel.add(textField);
		tmpPanel.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		//tmpPanel.add(Box.createHorizontalGlue());

		inputPanel.add(tmpPanel);
		inputPanel.add(Box.createRigidArea(new Dimension(0, 5)));
		inputPanel.add(maskThresholdSlider);
		inputPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		//just a friendly label panel.
		JPanel labelPanel = new JPanel();
		labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.LINE_AXIS));
		JLabel alabel = new JLabel(
				"<html>Pupil Response values below this threshold are ignored<br>during visualization and statistics evaluation.");
		labelPanel.add(alabel);
		labelPanel.add(Box.createHorizontalGlue());
		labelPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		//initialize the ok and cancel buttons
		JButton okBu = new JButton("OK");
		okBu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				dataEntered = true;
				PupilMaskDialog.this.setVisible(false);
			}

		});
		JButton cancelBu = new JButton("Cancel");
		cancelBu.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				PupilMaskDialog.this.setVisible(false);
				//get rid of preview
				PupilMaskDialog.this.handler.previewMaskSettings(oldMask);
			}

		});

		//the button panel holds the ok and cancel buttons
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
		buttonPanel.add(okBu);
		buttonPanel.add(cancelBu);

		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.PAGE_AXIS));
		this.getContentPane().add(labelPanel);
		this.getContentPane().add(Box.createRigidArea(new Dimension(0, 5)));
		this.getContentPane().add(inputPanel);
		this.getContentPane().add(buttonPanel);

		this.pack();
	}

	public float getMask() {
		return (float) maskThresholdSlider.getValue() / 100;
	}

	public boolean isDataEntered() {
		return dataEntered;
	}

}