/*
 * Created on Jul 17, 2004
 * SessionHandler.java
 * 
 * Project: eyetrack_cvs
 */
package pupil;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.tree.*;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import session.SessionHandler;
import task.ReadTaskXML;
import task.TaskBlock;
import video.VideoDialog;
import video.VideoView;
import view.NotesView;
import view.StatsView;
import view.View;
import view.ViewPanel;



/**
 * @author chris
 *  
 */
public class PupilSessionHandler extends SessionHandler {
	/** the data for this session */
	private PupilSessionData sessionData;

	/** the stats view* */
	private View statsView;

	/** the notes view* */
	private View notesView;

	/** tabbed pane for views to go */
	private JTabbedPane tabbedPane;

	/** the radio button group for the global menu* */
	private ButtonGroup globalMenuGroup;

	/** the pupil trial global menu items* */
	private JMenuItem[] pupilItems;

	/** the task model global menu items* */
	private JMenuItem[] taskItems;

	/** the video frame* */
	private View videoView;

	/** the currently selected view Panel* */
	private ViewPanel selectedView = null;

	/** the windows menu* */
	private JMenuItem[] windowItems;

	// Bailey changes
	private JSplitPane splitPane;
	private JTree tree;
	private DefaultMutableTreeNode top;
	private DefaultMutableTreeNode category;



	/**
	 * default constructor
	 * 
	 * @param owner -
	 *            the parent frame
	 */
	public PupilSessionHandler(JFrame owner) {
		super(owner);


		//initialize the session data
		sessionData = new PupilSessionData();

		//set up the tabbed pane for the vis panels
		tabbedPane = new JTabbedPane(SwingConstants.BOTTOM);
		tabbedPane.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				if (tabbedPane.getTabCount() == 0)
					return;

				//feed the statsView notes back into the previous
				//view handler
				if (selectedView != null && statsView != null) {
					selectedView.setNotes(((NotesView) notesView).getNotes());
				}

				//set the new selected View...
				selectedView = (PupilViewPanel) tabbedPane
						.getSelectedComponent();
				if (selectedView == null)
					return;

				//tell the stats panel and video panel what's up
				if (statsView != null) {
					statsView.setGlobal(selectedView.getMinGlobal(),
							selectedView.getMaxGlobal());
					statsView.setLocal(selectedView.getMinLocal(), selectedView
							.getMaxLocal());
					statsView.setSelection(selectedView.getMinSelection(),
							selectedView.getMaxSelection());

					((NotesView) notesView).setNotes(selectedView.getNotes());
				}

				if (videoView != null) {
					videoView.setGlobal(selectedView.getMinGlobal(),
							selectedView.getMaxGlobal());
					videoView.setLocal(selectedView.getMinLocal(), selectedView
							.getMaxGlobal());
					videoView.setSelection(selectedView.getMinSelection(),
							selectedView.getMaxSelection());
				}

				//update the global menu items...
				//get the global menu name
				String text = selectedView.getGlobalMenuText();
				if (text == null)
					return;

				//see if it exists...
				if (pupilItems != null) {
					for (int i = 0; i < pupilItems.length; i++) {
						if (pupilItems[i].getText().equals(text)) {
							pupilItems[i].setSelected(true);
							return;
						}
					}
				}
				if (taskItems != null) {
					for (int i = 0; i < taskItems.length; i++) {
						if (taskItems[i].getText().equals(text)) {
							taskItems[i].setSelected(true);
							return;
						}
					}
				}
			}
		});



// Bailey hack
splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
splitPane.setRightComponent(tabbedPane);

top = new DefaultMutableTreeNode("User Data");
tree = new JTree(top);
category = new DefaultMutableTreeNode("User 1");
top.add(category);
category = new DefaultMutableTreeNode("User 2");
top.add(category);
category = new DefaultMutableTreeNode("User 3");
top.add(category);
category = new DefaultMutableTreeNode("User 4");
top.add(category);
		splitPane.setLeftComponent(tree);



		this.setLayout(new GridLayout(1, 1));
		this.add(splitPane);
		this.setVisible(true);

		//add a boring default view
		addView();

		//initialize the global button group
		globalMenuGroup = new ButtonGroup();

		//add the stats frame...
		statsView = new StatsView(owner, "Statistics");
		((StatsView) statsView).setResizable(true);

		//also, we want to listen for frame actions...
		((StatsView) statsView).addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				windowItems[2].setText("Statistics");
			}
		});

		//add the notes frame...
		notesView = new NotesView(owner, "Notes");
		((NotesView) notesView).setResizable(true);

		//we want to listen for this frame's actions...
		((NotesView) notesView).addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				windowItems[3].setText("Notes");
			}
		});

		//try to add the video frame
		try {
			videoView = new VideoView(owner, this);
			((VideoView) videoView).setResizable(true);

			//we want to listen for close action
			((VideoView) videoView).addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					windowItems[4].setText("Video");
				}
			});
		} catch (Exception e) {
			videoView = null;
		}

	}

	/**
	 * method adds a default view to this session
	 *  
	 */
	public void addView() {
		//create blank view, add it to the tabbedpane
		PupilViewPanel tmpView = new PupilViewPanel(this);

		//add pupil data if it exists...
		if (sessionData.getPupilData() != null) {
			tmpView.setPupilData(sessionData.getPupilData(), sessionData
					.getBaseLine(), sessionData.getMinPCPS(), sessionData
					.getMaxPCPS());
			tmpView.setPupilMask(sessionData.getPupilMask(), true);
		}

		//add task data if it exists...
		if (sessionData.getTaskModel() != null)
			tmpView.setTaskModel(sessionData.getTaskModel());

		//set the time/selection data from any previously open view
		if (tabbedPane.getTabCount() > 0) {
			tmpView.setGlobal(selectedView.getMinGlobal(), selectedView
					.getMaxGlobal());
			tmpView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
			tmpView.setSelection(selectedView.getMinSelection(), selectedView
					.getMaxSelection());

			tmpView.setGlobalMenuText(selectedView.getGlobalMenuText());
		}

		//get the appropriate label
		sessionData.incHistoricViewCount();
		tmpView.setName("View" + sessionData.getHistoricViewCount());

		tabbedPane.addTab(tmpView.getName(), null, tmpView, tmpView.getName());
	}

	/**
	 * method removes the currently active view
	 *  
	 */
	public void removeView() {
		//make sure we can...
		if (tabbedPane.getComponentCount() > 0)
			tabbedPane.remove(tabbedPane.getSelectedIndex());
	}

	/**
	 * gets the selected view name. returns null if no views are open
	 * 
	 * @return
	 */
	public String getSelectedViewName() {
		if (selectedView != null)
			return selectedView.getName();
		else
			return null;
	}

	/**
	 * method sets the name of the currently selected view
	 * 
	 * @param name
	 */
	public void setSelectedViewName(String name) {
		if (selectedView != null) {
			tabbedPane.setTitleAt(tabbedPane.getSelectedIndex(), name);
			selectedView.setName(name);
		}
	}

	/**
	 * get the minimum selection time of the selected view
	 * 
	 * @return
	 */
	public long getSelectedViewMinSelect() {
		if (selectedView != null)
			return selectedView.getMinSelection();
		else
			return -1;
	}

	/**
	 * get the maximum selected time of the selected view
	 * 
	 * @return
	 */
	public long getSelectedViewMaxSelect() {
		if (selectedView != null)
			return selectedView.getMaxSelection();
		else
			return -1;
	}

	/**
	 * get the minimum focus time of the selected view
	 * 
	 * @return
	 */
	public long getSelectedViewMinLocal() {
		if (selectedView != null)
			return selectedView.getMinLocal();
		else
			return -1;
	}

	/**
	 * get the maximum focus time of the selected view
	 * 
	 * @return
	 */
	public long getSelectedViewMaxLocal() {
		if (selectedView != null)
			return selectedView.getMaxLocal();
		else
			return -1;
	}

	/**
	 * returns the currently available number of views...
	 * 
	 * @return
	 */
	public int getNumberOfViews() {
		return tabbedPane.getComponentCount();
	}

	/**
	 * method writes data to the session file stored in sessionData
	 * 
	 * @throws Exception
	 */
	public void writeDataToFile() throws Exception {
		//make sure a file exists to write to
		File file = getSessionFile();
		if (file == null)
			throw new Exception("No File");

		//write statsframe notes into the selected view
		if (statsView != null && selectedView != null) {
			selectedView.setNotes(((NotesView) notesView).getNotes());
		}

		//import data from views...
		sessionData.getViews().clear();
		for (int i = 0; i < tabbedPane.getTabCount(); i++)
			sessionData.getViews().add(
					((PupilViewPanel) tabbedPane.getComponent(i))
							.exportSettings());

		//write data to file
		FileOutputStream fo = new FileOutputStream(file);
		ObjectOutputStream so = new ObjectOutputStream(fo);
		so.writeObject(sessionData);
		so.flush();
		so.close();
	}

	/**
	 * writes data to the parameter file
	 * 
	 * @param file -
	 *            the file to write out to
	 * @throws Exception
	 */
	public void writeDataToFile(File file) throws Exception {
		setSessionFile(file);
		writeDataToFile();
	}

	/**
	 * reads data from given file. saves data into session.
	 * 
	 * @param file
	 *            the file to read from
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public void readDataFromFile(File file) throws IOException,
			ClassNotFoundException {
		FileInputStream fi = new FileInputStream(file);
		ObjectInputStream si = new ObjectInputStream(fi);
		PupilSessionData tmpSession = (PupilSessionData) si.readObject();
		si.close();

		//as a precaution...
		setSessionFile(file);
		sessionData = tmpSession;

		//clear out any existing views being displayed
		tabbedPane.removeAll();

		//give data to the statsframe
		if (statsView != null) {
			((StatsView) statsView).setPupilData(sessionData.getPupilData());
			((StatsView) statsView).setBaseline(sessionData.getBaseLine());
			((StatsView) statsView).setTaskModel(sessionData.getTaskModel());
		}

		//add the old views
		for (int i = 0; i < sessionData.getViews().size(); i++) {
			PupilViewPanel tmpView = new PupilViewPanel(
					PupilSessionHandler.this);

			//add pupil data if it exists...
			if (sessionData.getPupilData() != null) {
				tmpView.setPupilData(sessionData.getPupilData(), sessionData
						.getBaseLine(), sessionData.getMinPCPS(), sessionData
						.getMaxPCPS());
				tmpView.setPupilMask(sessionData.getPupilMask(), true);
			}

			//add task model if it exists...
			if (sessionData.getTaskModel() != null)
				tmpView.setTaskModel(sessionData.getTaskModel());

			//give the view it's settings to use
			tmpView.importSettings((Vector) sessionData.getViews().get(i));

			tabbedPane.addTab(tmpView.getName(), null, tmpView, tmpView
					.getName());
		}

		//check if video exists...
		if (sessionData.getVideoFile() != null) {
			try {
				//create the new video...
				View newVideoFrame = new VideoView(getOwner(), this);
				((VideoView) newVideoFrame).setVideo(sessionData.getVideoFile()
						.getAbsolutePath());
				((VideoView) newVideoFrame).setVideo(null);
				((VideoView) newVideoFrame).hide();
				videoView = newVideoFrame;
				((VideoView) videoView).setOffset(sessionData.getVideoOffset());
				((VideoView) videoView).addWindowListener(new WindowAdapter() {
					public void windowClosing(WindowEvent e) {
						windowItems[4].setText("Show Video");
					}
				});
				setShowVideo(false);

				videoView.setLocal(selectedView.getMinLocal(), selectedView
						.getMaxLocal());
				videoView.setSelection(selectedView.getMinSelection(),
						selectedView.getMaxSelection());
				videoView.setGlobal(selectedView.getMinGlobal(), selectedView
						.getMaxGlobal());
			} catch (Exception e) {
				//uh oh, video doesn't exist...
				JOptionPane
						.showMessageDialog(
								getOwner(),
								"<html>Sorry, unable to load the video <br>"
										+ sessionData.getVideoFile()
												.getAbsolutePath()
										+ "<br>The video may have been moved or may no longer exist.",
								"Cannot Load Video", JOptionPane.ERROR_MESSAGE);
				sessionData.setVideoFile(null);
				try {
					((VideoView) videoView).setVideo(null);
				} catch (Exception ex) {
				}
				setShowVideo(false);
			}
		} else {
			try {
				((VideoView) videoView).setVideo(null);
			} catch (Exception ex) {
			}
			setShowVideo(false);
		}
		tabbedPane.repaint();
	}

	/**
	 * method imports pupil data into the session
	 * 
	 * @param file -
	 *            the file from which the data was imported
	 * @throws IOException
	 * @throws NumberFormatException
	 */
	public void importPupilData(File file) throws NumberFormatException,
			IOException {
		//read the file witht the pupil data
		ReadPupil pupilReader = new ReadPupil(file.getAbsolutePath());
		pupilReader.read();

		//ask the user what pupil they want to use
		Object[] options = { "Left", "Right" };
		int n = JOptionPane.showOptionDialog(getOwner(),
				"What eye would you like to read in for this session?",
				"Importing", JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
		int whichEye;
		switch (n) {
		case 0:
			whichEye = ReadPupil.LEFT;
			break;
		default:
			whichEye = ReadPupil.RIGHT;
			break;
		}

		//get the baseline test interval from the user
		BaselineDialog bdia = new BaselineDialog(getOwner(), pupilReader
				.getMin(whichEye), pupilReader.getMax(whichEye));
		bdia.setModal(true);
		bdia.setVisible(true);

		//only proceed if valid data was given
		if (!bdia.isDataEntered())
			return;

		//get the baseline
		float base;
		if ((base = bdia.getBaseLine()) == -1) {
			double minBaseTime = bdia.getMin();
			double maxBaseTime = bdia.getMax();

			base = ReadPupil.calculateBaseLine(pupilReader.getData(
					(long) minBaseTime, (long) maxBaseTime, whichEye));
		}

		//now we have everything we need. import pupil data into
		//session
		sessionData.setPupilData(ReadPupil.prunePupilData(pupilReader.getData(
				ReadPupil.ALL_TRIALS, whichEye)));
		sessionData.setPupilFile(file);
		sessionData.setEye(whichEye);
		sessionData.setBaseLine(base);
		float minPCPS = ReadPupil.calculateMinPCPS(sessionData.getPupilData(),
				base);
		sessionData.setMinPCPS(minPCPS);
		sessionData.setMaxPCPS(ReadPupil.calculateMaxPCPS(sessionData
				.getPupilData(), base));
		sessionData.setTrialData(pupilReader.getTrialData());
		sessionData.setPupilMask(minPCPS);

		//set pupil data for all views
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			PupilViewPanel PH = (PupilViewPanel) tabbedPane.getComponent(i);
			PH.setPupilData(sessionData.getPupilData(), sessionData
					.getBaseLine(), sessionData.getMinPCPS(), sessionData
					.getMaxPCPS());
			PH.setPupilMask(sessionData.getPupilMask(), true);

		}
		tabbedPane.repaint();

		//set pupil data for stats frame
		if (statsView != null) {
			((StatsView) statsView).setPupilData(sessionData.getPupilData());
			((StatsView) statsView).setBaseline(sessionData.getBaseLine());

			//align the stats frame with the selected view
			statsView.setGlobal(selectedView.getMinGlobal(), selectedView
					.getMaxGlobal());
			statsView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
			statsView.setSelection(selectedView.getMinSelection(), selectedView
					.getMaxSelection());
		}

		//align the video with the data
		if (videoView != null) {
			videoView.setGlobal(selectedView.getMinGlobal(), selectedView
					.getMaxGlobal());
			videoView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
			videoView.setSelection(selectedView.getMinSelection(), selectedView
					.getMaxSelection());
		}

	}

	/**
	 * method imports task model into the session
	 * 
	 * @param file -
	 *            the file read from
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 */
	public void importTaskModel(File file) throws SAXException, IOException,
			ParserConfigurationException {

		//read the task model
		ReadTaskXML taskReader;
		taskReader = new ReadTaskXML(file.getAbsolutePath());
		taskReader.read();

		sessionData.setTaskModel(taskReader.getData());
		sessionData.setTaskFile(file);

		//set task model for all views
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			PupilViewPanel PH = (PupilViewPanel) tabbedPane.getComponent(i);
			PH.setTaskModel(sessionData.getTaskModel());

			//set the global menu text...
			if (sessionData.getPupilData() == null) {

			}

		}
		tabbedPane.repaint();

		//set the task model for the stats frame
		if (statsView != null) {
			((StatsView) statsView).setTaskModel(sessionData.getTaskModel());

			//align the stats frame with the selected view
			statsView.setGlobal(selectedView.getMinGlobal(), selectedView
					.getMaxGlobal());
			statsView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
			statsView.setSelection(selectedView.getMinSelection(), selectedView
					.getMaxSelection());
		}

		//align the video with the data
		if (videoView != null) {
			videoView.setGlobal(selectedView.getMinGlobal(), selectedView
					.getMaxGlobal());
			videoView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
			videoView.setSelection(selectedView.getMinSelection(), selectedView
					.getMaxSelection());
		}
	}

	/**
	 * method imports video into the session
	 * 
	 * @param file -
	 *            the video file
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws QTException
	 */
	public void importVideo(File file) throws SecurityException,
			IllegalArgumentException, NoSuchMethodException,
			ClassNotFoundException, IllegalAccessException,
			InvocationTargetException /* throws QTException */{
		//add video to video frame
		VideoView newVideoFrame = new VideoView(getOwner(), this);
		try {
			newVideoFrame.setVideo(file.getAbsolutePath());
		} catch (Exception e) {
			e.printStackTrace();
		}

		//get the offset for the video
		VideoDialog vdia = new VideoDialog(getOwner());
		vdia.setModal(true);
		vdia.setVisible(true);

		//no offset? get out
		if (!vdia.dataEntered())
			return;

		//get rid of the old video
		try {
			((VideoView) videoView).setVideo(null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		videoView = newVideoFrame;
		((VideoView) videoView).addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				windowItems[4].setText("Video");
			}
		});
		((VideoView) videoView).setVisible(true);

		//open the video frame
		setShowVideo(true);

		//save the video file and offset
		sessionData.setVideoFile(file);
		sessionData.setVideoOffset(vdia.getOffset());

		//set the offset.
		((VideoView) videoView).setOffset(vdia.getOffset());

		//set up the video with the correct times...
		videoView.setLocal(selectedView.getMinLocal(), selectedView
				.getMaxLocal());
		videoView.setSelection(selectedView.getMinSelection(), selectedView
				.getMaxSelection());
		videoView.setGlobal(selectedView.getMinGlobal(), selectedView
				.getMaxGlobal());
		setShowVideo(true);
	}

	/**
	 * method purges all saved settings from session. resulting object resembles
	 * newly constructed default session object
	 */
	public void purge() {
		//get a new session
		sessionData = new PupilSessionData();

		//add a boring default view
		tabbedPane.removeAll();
		addView();

		//clear out the stats frame
		statsView.purge();
		((StatsView) statsView).repaint();

		//get rid of the video data/hide it
		if (videoView != null) {
			setShowVideo(false);
			try {
				videoView.purge();
			} catch (Exception ex) {
			}
		}
	}

	/**
	 * method returns an array of menu items for the purposes of setting pupil
	 * global menu items...
	 * 
	 * @return
	 */
	public JMenuItem[] getPupilTrialMenuItems() {
		if (sessionData.getTrialData() == null)
			return null;

		//check if pupilItems needs to be removed...
		if (pupilItems != null) {
			for (int i = 0; i < pupilItems.length; i++)
				globalMenuGroup.remove(pupilItems[i]);

		} else {
			pupilItems = new GlobalMenuItemListener[sessionData.getTrialData().length + 1];
		}

		pupilItems[0] = new GlobalMenuItemListener(
				sessionData.getTrialData()[0][0],
				sessionData.getTrialData()[sessionData.getTrialData().length - 1][1],
				"All Trials");
		globalMenuGroup.add(pupilItems[0]);

		//select if no other menu exists....
		if (taskItems == null)
			pupilItems[0].setSelected(true);

		for (int i = 0; i < sessionData.getTrialData().length; i++) {
			pupilItems[i + 1] = new GlobalMenuItemListener(sessionData
					.getTrialData()[i][0], sessionData.getTrialData()[i][1],
					"Trial " + (i + 1));
			globalMenuGroup.add(pupilItems[i + 1]);
		}

		//check if the selected tab has a predisposition....
		if (selectedView != null) {
			//see if it exists...
			if (pupilItems != null) {
				for (int i = 0; i < pupilItems.length; i++) {
					if (((GlobalMenuItemListener) pupilItems[i]).getMinTime() == selectedView
							.getMinGlobal()
							&& ((GlobalMenuItemListener) pupilItems[i])
									.getMaxTime() == selectedView
									.getMaxGlobal()) {
						pupilItems[i].setSelected(true);
						break;
					}
				}
			}
		}

		return pupilItems;
	}

	/**
	 * method returns an array of menu items for the purposes of setting task
	 * model global menu items...
	 * 
	 * @return
	 */
	public JMenuItem[] getTaskMenuItems() {
		if (sessionData.getTaskModel() == null)
			return null;
		if (taskItems != null) {

		} else {
			taskItems = new GlobalMenuItemListener[((Vector) sessionData
					.getTaskModel().get(0)).size() + 1];
		}

		//the highest level tasks
		Vector highTask = (Vector) sessionData.getTaskModel().get(0);

		TaskBlock firstBlock = (TaskBlock) highTask.firstElement();
		TaskBlock lastBlock = (TaskBlock) highTask.lastElement();

		taskItems[0] = new GlobalMenuItemListener(firstBlock.getStartTime(),
				lastBlock.getEndTime(), "Entire Task Model");
		globalMenuGroup.add(taskItems[0]);

		//if no other menu exists, select this item
		if (pupilItems == null)
			taskItems[0].setSelected(true);

		for (int i = 0; i < highTask.size(); i++) {
			TaskBlock tmp = (TaskBlock) highTask.get(i);
			taskItems[i + 1] = new GlobalMenuItemListener(tmp.getStartTime(),
					tmp.getEndTime(), tmp.getName());
			globalMenuGroup.add(taskItems[i + 1]);
		}

		//check if the selected tab has a predisposition....
		if (selectedView != null) {
			//see if it exists...
			if (taskItems != null) {
				for (int i = 0; i < taskItems.length; i++) {
					if (((GlobalMenuItemListener) taskItems[i]).getMinTime() == selectedView
							.getMinGlobal()
							&& ((GlobalMenuItemListener) taskItems[i])
									.getMaxTime() == selectedView
									.getMaxGlobal()) {
						taskItems[i].setSelected(true);
						break;
					}
				}
			}
		}

		return taskItems;
	}

	/**
	 * 
	 * @author chris lisener class for global menu items
	 */
	private class GlobalMenuItemListener extends JRadioButtonMenuItem implements
			ActionListener {
		/** the min time associated with this global* */
		private long minTime;

		/** the max time associated with this global* */
		private long maxTime;

		/** the name of this menu item* */
		private String name;

		/**
		 * the default constructor...
		 *  
		 */
		public GlobalMenuItemListener() {
			addActionListener(this);
		}

		/**
		 * better constructor
		 * 
		 * @param minTime
		 * @param maxTime
		 * @param name
		 */
		public GlobalMenuItemListener(long minTime, long maxTime, String name) {
			super(name);
			addActionListener(this);
			this.minTime = minTime;
			this.maxTime = maxTime;
			this.name = name;
		}

		public void actionPerformed(ActionEvent e) {
			//set the global times of the current view...
			PupilSessionHandler.this.setSelectedViewGlobal(minTime, maxTime,
					null);
			selectedView.setGlobalMenuText(name);
		}

		/**
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}

		/**
		 * @param name
		 *            The name to set.
		 */
		public void setName(String name) {
			this.name = name;
		}

		/**
		 * @return Returns the maxTime.
		 */
		public long getMaxTime() {
			return maxTime;
		}

		/**
		 * @return Returns the minTime.
		 */
		public long getMinTime() {
			return minTime;
		}

		/**
		 * method sets the time for this global choice
		 * 
		 * @param minTime
		 * @param maxTime
		 */
		public void setTimes(long minTime, long maxTime) {
			this.minTime = minTime;
			this.maxTime = maxTime;
		}
	}

	/**
	 * method routes a set focus command to all active modules controlled by the
	 * session handler. if the object calling this method is not null and is not
	 * active, the command will not be carried out.
	 * 
	 * @param minTime -
	 *            the min Focus
	 * @param maxTime -
	 *            the max focus time
	 * @param m -
	 *            the object calling this method
	 */
	public void setSelectedViewLocal(long minTime, long maxTime, View v) {
		//make sure m isn't some view handler that isn't selected...
		if (v != null && v instanceof ViewPanel) {
			if (v != selectedView)
				return;
		} else {
			//update selected view?
			if (selectedView != null)
				selectedView.setLocal(minTime, maxTime);
		}

		//update stats?
		if (v != statsView && statsView != null)
			statsView.setLocal(minTime, maxTime);

		//update video?
		if (v != videoView && videoView != null)
			videoView.setLocal(minTime, maxTime);
	}

	/**
	 * method routes a set focus command to all active modules controlled by the
	 * session handler. if the object calling this method is not null and is not
	 * active, the command will not be carried out.
	 * 
	 * @param minTime -
	 *            the min Focus
	 * @param maxTime -
	 *            the max focus time
	 * @param m -
	 *            the object calling this method
	 */
	public void setSelectedViewGlobal(long minTime, long maxTime, View v) {
		//make sure m isn't some view handler that isn't selected...
		if (v != null && v instanceof ViewPanel) {
			if (v != selectedView)
				return;
		} else {
			//update selected view?
			if (selectedView != null) {
				selectedView.setGlobal(minTime, maxTime);
				//do we also need to update the focus?
				if (selectedView.getMinLocal() < minTime
						|| selectedView.getMinLocal() > maxTime
						|| selectedView.getMaxLocal() < minTime
						|| selectedView.getMaxLocal() > maxTime) {
					selectedView.setLocal(minTime, maxTime);
				}
			}
		}

		//update stats?
		if (v != statsView && statsView != null) {
			statsView.setGlobal(minTime, maxTime);
			statsView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
		}

		//update video?
		if (v != videoView && videoView != null) {
			videoView.setGlobal((long) minTime, (long) maxTime);
			videoView.setLocal(selectedView.getMinLocal(), selectedView
					.getMaxLocal());
		}
	}

	/**
	 * method routes a set focus command to all active modules controlled by the
	 * session handler. if the object calling this method is not null and is not
	 * active, the command will not be carried out.
	 * 
	 * @param minTime -
	 *            the min Focus
	 * @param maxTime -
	 *            the max focus time
	 * @param m -
	 *            the object calling this method
	 */
	public void setSelectedViewSelection(long minTime, long maxTime, View v) {
		//make sure m isn't some view handler that isn't selected...
		//if(m!=null && m instanceof ViewHandlerDesktop) {
		if (v != null && v instanceof PupilViewPanel) {
			if (v != selectedView)
				return;
		} else {
			//update selected view?
			if (selectedView != null)
				selectedView.setSelection((long) minTime, (long) maxTime);
		}

		//update stats?
		if (v != statsView && statsView != null)
			statsView.setSelection((long) minTime, (long) maxTime);

		//update video?
		if (v != videoView && videoView != null)
			videoView.setSelection((long) minTime, (long) maxTime);
	}

	/**
	 * method routes a set focus command to all active modules controlled by the
	 * session handler. if the object calling this method is not null and is not
	 * active, the command will not be carried out.
	 * 
	 * @param minTime -
	 *            the min Focus
	 * @param maxTime -
	 *            the max focus time
	 * @param m -
	 *            the object calling this method
	 */
	public void previewSelectedViewLocal(long minTime, long maxTime, View v) {
		//make sure m isn't some view handler that isn't selected...
		//if(m!=null && m instanceof ViewHandlerDesktop) {
		if (v != null && v instanceof PupilViewPanel) {
			if (v != selectedView)
				return;
		} else {
			//update selected view?
			if (selectedView != null) {
				selectedView.previewLocal(minTime, maxTime);
			}
		}

		//update stats?
		if (v != statsView && statsView != null)
			statsView.previewLocal(minTime, maxTime);

		//update video?
		if (v != videoView && videoView != null) {
			videoView.previewLocal(minTime, maxTime);
		}
	}

	/**
	 * method routes a set focus command to all active modules controlled by the
	 * session handler. if the object calling this method is not null and is not
	 * active, the command will not be carried out.
	 * 
	 * @param minTime -
	 *            the min Focus
	 * @param maxTime -
	 *            the max focus time
	 * @param m -
	 *            the object calling this method
	 */
	public void previewSelectedViewSelection(long minTime, long maxTime, View v) {
		//make sure m isn't some view handler that isn't selected...
		if (v != null && v instanceof PupilViewPanel) {
			if (v != selectedView)
				return;
		} else {
			//update selected view?
			if (tabbedPane != null && selectedView != null) {
				selectedView.previewSelection(minTime, maxTime);
			}
		}

		//update stats?
		if (v != statsView && statsView != null)
			statsView.previewSelection(minTime, maxTime);

		//update video?
		if (v != videoView && videoView != null)
			videoView.previewSelection(minTime, maxTime);
	}

	/**
	 * @param show
	 *            true if stats should be shown
	 */
	public void setShowStats(boolean show) {
		setShowFrame((StatsView) statsView, show);
	}

	/**
	 * 
	 * @return is stats shown
	 */
	public boolean isShowStats() {
		if (statsView == null)
			return false;
		else
			return ((StatsView) statsView).isVisible();
	}

	/**
	 * set whether the frame param is set or not
	 * 
	 * @param f
	 * @param isShown
	 */
	private void setShowFrame(JDialog f, boolean isShown) {
		if (f != null) {

			if (isShown) {
				//make sure frame isn't closed or iconified.
				if (!f.isVisible()) {
					//"unclose" it and make visible
					f.setVisible(true);

				}
			} else {
				if (f.isVisible()) {
					f.setVisible(false);
				}
			}
		}
	}

	/**
	 * 
	 * @param show
	 *            true if video window shown
	 */
	public void setShowVideo(boolean show) {
		setShowFrame((VideoView) videoView, show);

		if (show)
			windowItems[4].setText("Hide Video");
		else
			windowItems[4].setText("Video");
	}

	/**
	 * 
	 * @param show
	 *            is notes shown
	 */
	public void setShowNotes(boolean show) {
		setShowFrame((NotesView) notesView, show);
	}

	/**
	 * 
	 * @return is notes shown
	 */
	public boolean isShowVideo() {
		if (videoView == null)
			return false;
		else
			return ((VideoView) videoView).isVisible();
	}

	/**
	 * 
	 * @return true if we have pupil data, false otherwise
	 */
	public boolean havePupilData() {
		if (sessionData.getPupilData() == null)
			return false;
		else
			return true;
	}

	/**
	 * 
	 * @return true if we have task model, false other wise
	 */
	public boolean haveTaskData() {
		if (sessionData.getTaskModel() == null)
			return false;
		else
			return true;
	}

	/**
	 * true if we have video data, false otherwise
	 * 
	 * @return
	 */
	public boolean haveVideoData() {
		if (sessionData.getVideoFile() == null)
			return false;
		else
			return true;
	}

	/**
	 * 
	 * @return is shown notes
	 */
	public boolean isShowNotes() {
		if (notesView == null)
			return false;
		else
			return ((NotesView) notesView).isVisible();
	}

	/**
	 * @param windowItems
	 *            The windowItems to set.
	 */
	public void setWindowItems(JMenuItem[] windowItems) {
		this.windowItems = windowItems;
	}

	/**
	 * method sets the playhead of the selected view
	 * 
	 * @param playhead
	 */
	public void setPlayHead(double playhead) {
		if (selectedView != null) {
			((PupilViewPanel) selectedView).setPlayHead(playhead);
			selectedView.repaint();
		}
	}

	/**
	 * calling this method pops up a dialog box to get the pupil mask
	 */
	public void dispPupilMaskSettings() {
		//no data? get out.
		if (sessionData.getPupilData() == null)
			return;

		//get the pupil mask
		PupilMaskDialog pmdia = new PupilMaskDialog(getOwner(), this,
				sessionData.getMinPCPS(), sessionData.getMaxPCPS(), sessionData
						.getPupilMask());
		pmdia.setModal(true);
		pmdia.setVisible(true);

		//was there even anything new?
		if (!pmdia.isDataEntered()
				|| (pmdia.getMask() == sessionData.getPupilMask()))
			return;

		//a new mask value! store this value
		sessionData.setPupilMask(pmdia.getMask());

		//also store this value in all of the views
		//set pupilData for all views
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			PupilViewPanel PH = (PupilViewPanel) tabbedPane.getComponent(i);
			PH.setPupilMask(pmdia.getMask(), true);
		}
		tabbedPane.repaint();

		//update the stats frame
		((StatsView) statsView).setMaskThreshold(pmdia.getMask());
	}






	public void dispPupilFilterSettings() {
		//no data? get out.
		if (sessionData.getPupilData() == null)
			return;

		//get the pupil mask
		SmoothingDialog sd = new SmoothingDialog(getOwner(), 5);
		sd.setModal(true);
		sd.setVisible(true);

		//was there even anything new?
		if (sd.getSmoothingValue() > 0) {
			smoothData(sd.getSmoothingValue());
		}

	}



	public void smoothData(int amount) {
		Vector data = sessionData.getPupilData();
		Vector newData = ReadPupil.meanFilter(data, amount);

		sessionData.setPupilData(newData);

		float minPCPS = ReadPupil.calculateMinPCPS(sessionData.getPupilData(),sessionData.getBaseLine());
		sessionData.setMinPCPS(minPCPS);
		sessionData.setMaxPCPS(ReadPupil.calculateMaxPCPS(sessionData
				.getPupilData(), sessionData.getBaseLine()));
		sessionData.setPupilMask(minPCPS);

		//set pupil data for all views
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			PupilViewPanel PH = (PupilViewPanel) tabbedPane.getComponent(i);
			PH.setPupilData(sessionData.getPupilData(), sessionData
					.getBaseLine(), sessionData.getMinPCPS(), sessionData
					.getMaxPCPS());
			PH.setPupilMask(sessionData.getPupilMask(), true);
		}
		tabbedPane.repaint();

	}





	public void previewMaskSettings(float mask) {
		if (sessionData.getPupilData() != null && selectedView != null) {
			((PupilViewPanel) selectedView).setPupilMask(mask, false);
			selectedView.repaint();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see session.SessionHandler#previewGlobal(long, long, view.View)
	 */
	public void previewSelectedViewGlobal(long minTime, long maxTime,
			View theCallingView) {
		// ....
	}

}