/*
 * Created on Sep 16, 2004
 * PupilViewPanel.java
 * 
 * Project: eyetrack_cvs
 */
package pupil;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.text.DecimalFormat;
import java.util.Vector;

import javax.swing.ToolTipManager;

import session.SessionHandler;
import task.TaskBlock;
import task.TaskView;
import view.GlobalVisWindowCompositor;
import view.GlobalWindowDecorator;
import view.LocalRulerDecorator;
import view.LocalWindowDecorator;
import view.TimeRuler;
import view.ViewPanel;
import view.VisWindow;

/**
 * @author chris
 *  
 */
public class PupilViewPanel extends ViewPanel implements Runnable {

	/** local task rectangle */
	private Rectangle localTaskRect;

	/** global task rectangle */
	private Rectangle globalTaskRect;

	/** local pupil rectangle */
	private Rectangle localPupilRect;

	/** global pupil rectangle */
	private Rectangle globalPupilRect;

	/** top local time ruler */
	private Rectangle topLocalTimeRulerRect;

	/** bottom local time ruler */
	private Rectangle botLocalTimeRulerRect;

	/** local task window */
	private VisWindow localTaskView;

	/** global task view */
	private VisWindow globalTaskView;

	/** local pupil view */
	private VisWindow localPupilView;

	private VisWindow globalPupilView;

	/** top local time ruler */
	private VisWindow topLocalTimeRuler;

	/** bottom local time ruler */
	private VisWindow botLocalTimeRuler;

	/** the y value by which the local and global views are seperated */
	private int ySeperator;

	/** the font for labels */
	private Font f;

	/** local text label */
	private String localText;

	/** global text label */
	private String globalText;

	/** decimal format for number labels */
	private DecimalFormat df;

	/** the data for this view */
	private PupilViewData perspectiveData;

	/** the thread responsible for animating the marquees */
	private Thread animationThread;

	/** the animation delay */
	private int animationDelay = 83;

	/** stroke used for marquee* */
	private BasicStroke dashed;

	/** pattern for marquee stroke* */
	private float dash1[] = { 10.0f };

	/** the boundary selector widget* */
	private BoundarySelector boundsSelector;

	/** the task model boundaries... */
	private Vector taskBoundaries = null;

	/** the levels to draw...* */
	private boolean[] drawLevels;

	/** the playhead time* */
	private double playHead = -1;

	/** the composited global windows */
	private VisWindow globalComposite;

	private class ViewPanelListener implements MouseMotionListener,
			MouseListener {

		/** action=nothing */
		public final static int NOTHING = 0;

		/** action=zooming in local task view */
		public final static int SETTING_ZOOM_BY_LOCAL_TASK = 1;

		/** action=zooming in local pupil view */
		public final static int SETTING_ZOOM_BY_LOCAL_PUPIL = 2;

		/** action=zooming in a global view */
		public final static int SETTING_ZOOM_BY_GLOBAL = 3;

		/** action=moving the zoom */
		public final static int MOVING_FOCUS = 4;

		/** action=resizing the zoom */
		public final static int RESIZE_FOCUS = 5;

		/** action=selecting in the local task */
		public final static int SETTING_SEL_BY_LOCAL_TASK = 6;

		/** action=selecting in the local pupil */
		public final static int SETTING_SEL_BY_LOCAL_PUPIL = 7;

		/** the action flag* */
		private int action = NOTHING;

		/** the saved time used for mouse listening...* */
		private long savedTime;

		/** boolean indicating that a preview is taking place* */
		private boolean previewing;

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
		 */
		public void mouseDragged(MouseEvent e) {
			//case by case breakdown...
			double mouseTime;
			switch (action) {
			case ViewPanelListener.NOTHING:
				return;
			case ViewPanelListener.SETTING_SEL_BY_LOCAL_TASK:
			//same as local pupil...
			case ViewPanelListener.SETTING_SEL_BY_LOCAL_PUPIL:
				//make sure we are previewing...
				previewing = true;

				mouseTime = localPupilView.getTime(e.getX());

				//cap the times by the local boundaries
				if (mouseTime < getMinLocal())
					mouseTime = getMinLocal();
				if (mouseTime > getMaxLocal())
					mouseTime = getMaxLocal();
				if (savedTime < mouseTime) {
					sessionHandler.previewSelectedViewSelection(savedTime,
							(long) Math.ceil(mouseTime), PupilViewPanel.this);
					previewSelection(savedTime, (long) Math.ceil(mouseTime));
				} else {
					sessionHandler.previewSelectedViewSelection(
							(long) mouseTime, savedTime, PupilViewPanel.this);
					previewSelection((long) mouseTime, savedTime);
				}

				PupilViewPanel.this.repaint();
				break;

			case ViewPanelListener.SETTING_ZOOM_BY_LOCAL_TASK:
			//same as local pupil...
			case ViewPanelListener.SETTING_ZOOM_BY_LOCAL_PUPIL:
				//make sure we are previewing...
				previewing = true;

				mouseTime = localPupilView.getTime(e.getX());
				//cap the times by the local boundaries
				if (mouseTime < getMinLocal())
					mouseTime = getMinLocal();
				if (mouseTime > getMaxLocal())
					mouseTime = getMaxLocal();
				if (savedTime < mouseTime) {
					sessionHandler.previewSelectedViewLocal(savedTime,
							(long) Math.ceil(mouseTime), PupilViewPanel.this);
					previewLocalTimeByLocal(savedTime, (long) Math
							.ceil(mouseTime));
				} else {
					sessionHandler.previewSelectedViewLocal((long) mouseTime,
							savedTime, PupilViewPanel.this);
					previewLocalTimeByLocal((long) mouseTime, savedTime);
				}

				PupilViewPanel.this.repaint();
				break;

			case ViewPanelListener.RESIZE_FOCUS:
			//same as set zoom by global...
			case ViewPanelListener.SETTING_ZOOM_BY_GLOBAL:
				//make sure we are previewing...
				previewing = true;

				mouseTime = globalComposite.getTime(e.getX());
				//	cap the times by the global boundaries
				if (mouseTime < getMinGlobal())
					mouseTime = getMinGlobal();
				if (mouseTime > getMaxGlobal())
					mouseTime = getMaxGlobal();
				if (savedTime < mouseTime) {
					sessionHandler.previewSelectedViewLocal(savedTime,
							(long) Math.ceil(mouseTime), PupilViewPanel.this);
					previewLocalTimeByGlobal(savedTime, (long) Math
							.ceil(mouseTime));
				} else {
					sessionHandler.previewSelectedViewLocal((long) mouseTime,
							savedTime, PupilViewPanel.this);
					previewLocalTimeByGlobal((long) mouseTime, savedTime);
				}

				PupilViewPanel.this.repaint();
				break;

			case ViewPanelListener.MOVING_FOCUS:
				//make sure we are previewing...
				previewing = true;

				mouseTime = globalPupilView.getTime(e.getX());

				long minPreviewFocus = (long) mouseTime - savedTime;
				long maxPreviewFocus = minPreviewFocus
						+ (getMaxLocal() - getMinLocal());

				if (minPreviewFocus < getMinGlobal()) {
					maxPreviewFocus = getMinGlobal()
							+ (maxPreviewFocus - minPreviewFocus);
					minPreviewFocus = getMinGlobal();
				}
				if (maxPreviewFocus > getMaxGlobal()) {
					minPreviewFocus = getMaxGlobal()
							- (maxPreviewFocus - minPreviewFocus);
					maxPreviewFocus = getMaxGlobal();
				}

				savedTime = (long) mouseTime - minPreviewFocus;

				sessionHandler.previewSelectedViewLocal(minPreviewFocus,
						maxPreviewFocus, PupilViewPanel.this);
				previewLocalTimeByGlobal(minPreviewFocus, maxPreviewFocus);
				repaint();

				break;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
		 */
		public void mouseMoved(MouseEvent e) {
			//do we care?
			if (perspectiveData.getPupilData() == null
					&& perspectiveData.getTaskBlocks() == null) {
				setCursor(Cursor.getDefaultCursor());
				return;
			}

			//where did we hit?
			//local task view?
			if (perspectiveData.getTaskBlocks() != null
					&& localTaskRect.contains(e.getPoint())) {
				setCursor(Cursor.getDefaultCursor());
				return;
			}

			//local pupil view?
			if (perspectiveData.getPupilData() != null
					&& localPupilRect.contains(e.getPoint())) {
				setCursor(Cursor.getDefaultCursor());
				return;
			}

			//global view?
			if ((perspectiveData.getPupilData() != null || perspectiveData
					.getTaskBlocks() != null)
					&& globalComposite.getDrawRegion().contains(e.getPoint())) {

				//figure out what kind of action this is...
				if ((e.getX() >= globalComposite.getXCoord(getMinLocal()) - 1 && e
						.getX() <= globalComposite.getXCoord(getMinLocal()) + 1)) {
					setCursor(Cursor
							.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
				} else if (e.getX() >= globalComposite.getXCoord(getMaxLocal()) - 1
						&& e.getX() <= globalComposite.getXCoord(getMaxLocal()) + 1) {
					setCursor(Cursor
							.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
				} else if (e.getX() > globalComposite.getXCoord(getMinLocal())
						&& e.getX() < globalComposite.getXCoord(getMaxLocal())) {
					setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				} else {
					setCursor(Cursor.getDefaultCursor());
				}
				return;
			}

			//nothing else...
			setCursor(Cursor.getDefaultCursor());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			//check if boundary widget was selected
			if (boundsSelector != null
					&& perspectiveData.getPupilData() != null
					&& perspectiveData.getTaskBlocks() != null) {
				int intersect = boundsSelector.checkIntersect(e);

				//no hit
				if (intersect == -1)
					return;

				//there must be a hit. repaint
				repaint();

				//alert the view and repaint the panel
				for (int i = 0; i <= intersect; i++) {
					toggleBoundary(i, true);
				}
				for (int i = intersect + 1; i < perspectiveData
						.getTaskLevelColors().length; i++) {
					toggleBoundary(i, false);
				}

			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		 */
		public void mouseEntered(MouseEvent arg0) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		 */
		public void mouseExited(MouseEvent e) {
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
		public void mousePressed(MouseEvent e) {
			//do we care?
			if (perspectiveData.getPupilData() == null
					&& perspectiveData.getTaskBlocks() == null) {
				action = ViewPanelListener.NOTHING;
				return;
			}

			//get the "time" where the mouse was pressed
			double mouseTime;

			//where did we hit?
			//local task view?
			if (perspectiveData.getTaskBlocks() != null
					&& localTaskRect.contains(e.getPoint())) {
				mouseTime = localTaskView.getTime(e.getPoint().x);

				//selection...
				if (e.isShiftDown()) {
					action = ViewPanelListener.SETTING_SEL_BY_LOCAL_TASK;
					savedTime = (long) mouseTime;
				} else {
					action = ViewPanelListener.SETTING_ZOOM_BY_LOCAL_TASK;
					savedTime = (long) mouseTime;
				}

				return;
			}

			//local pupil view?
			if (perspectiveData.getPupilData() != null
					&& localPupilRect.contains(e.getPoint())) {
				mouseTime = localPupilView.getTime(e.getPoint().x);

				//selection...
				if (e.isShiftDown()) {
					action = ViewPanelListener.SETTING_SEL_BY_LOCAL_PUPIL;
					savedTime = (long) mouseTime;
				} else {
					//zooming...
					action = ViewPanelListener.SETTING_ZOOM_BY_LOCAL_PUPIL;
					//minPreviewFocus=maxPreviewFocus=savedTime=(long)mouseTime;
					savedTime = (long) mouseTime;
				}

				PupilViewPanel.this.repaint();
				return;
			}

			//one of the global views?
			if ((perspectiveData.getPupilData() != null || perspectiveData
					.getTaskBlocks() != null)
					&& globalComposite.getDrawRegion().contains(e.getPoint())) {

				mouseTime = globalComposite.getTime(e.getPoint().x);

				//figure out what kind of action this is...
				if ((e.getX() >= globalComposite.getXCoord(getMinLocal()) - 1 && e
						.getX() <= globalComposite.getXCoord(getMinLocal()) + 1)) {
					//resize min...
					action = ViewPanelListener.RESIZE_FOCUS;
					savedTime = getMaxLocal();
				} else if (e.getX() >= globalComposite.getXCoord(getMaxLocal()) - 1
						&& e.getX() <= globalComposite.getXCoord(getMaxLocal()) + 1) {
					//resize max
					action = ViewPanelListener.RESIZE_FOCUS;
					savedTime = getMinLocal();
				} else if (e.getX() > globalComposite.getXCoord(getMinLocal())
						&& e.getX() < globalComposite.getXCoord(getMaxLocal())) {
					//move
					action = ViewPanelListener.MOVING_FOCUS;
					savedTime = (long) mouseTime - getMinLocal();
				} else {
					//must be trying to set something
					savedTime = (long) mouseTime;
					action = ViewPanelListener.SETTING_ZOOM_BY_GLOBAL;
				}

				return;
			}

			//nothing else...
			action = ViewPanelListener.NOTHING;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
		public void mouseReleased(MouseEvent e) {
			//case by case breakdown...
			double mouseTime;
			switch (action) {
			case ViewPanelListener.NOTHING:
				return;
			case ViewPanelListener.SETTING_SEL_BY_LOCAL_TASK:
			//same as local pupil...
			case ViewPanelListener.SETTING_SEL_BY_LOCAL_PUPIL:
				mouseTime = localPupilView.getTime(e.getX());

				//check if nothing was actually done..
				if (!previewing) {

					//for now, deselect the current selected region
					sessionHandler.setSelectedViewSelection(-1, -1,
							PupilViewPanel.this);
					setSelection(-1, -1);
					repaint();

					break;
				}

				//turn off previewing...
				previewing = false;
				previewSelection(-1, -1);

				//cap the times by the local boundaries
				if (mouseTime < getMinLocal())
					mouseTime = getMinLocal();
				if (mouseTime > getMaxLocal())
					mouseTime = getMaxLocal();
				if (savedTime < mouseTime) {
					setSelection(savedTime, (long) Math.ceil(mouseTime));
					sessionHandler.setSelectedViewSelection(savedTime,
							(long) mouseTime, PupilViewPanel.this);
				} else {
					setSelection((long) mouseTime, savedTime);
					sessionHandler.setSelectedViewSelection((long) mouseTime,
							savedTime, PupilViewPanel.this);
				}
				break;
			case ViewPanelListener.SETTING_ZOOM_BY_LOCAL_TASK:
			//same as local pupil...
			case ViewPanelListener.SETTING_ZOOM_BY_LOCAL_PUPIL:
				mouseTime = localPupilView.getTime(e.getX());

				//check if nothing was actually done...
				if (!previewing) {
					repaint();
					break;
				}

				//turn off previewing...
				previewing = false;
				previewLocalTimeByLocal(-1, -1);

				if (mouseTime < getMinLocal())
					mouseTime = getMinLocal();
				if (mouseTime > getMaxLocal())
					mouseTime = getMaxLocal();
				if (savedTime < mouseTime) {
					setLocal(savedTime, (long) Math.ceil(mouseTime));
					sessionHandler.setSelectedViewLocal(savedTime,
							(long) mouseTime, PupilViewPanel.this);
				} else {
					setLocal((long) mouseTime, savedTime);
					sessionHandler.setSelectedViewLocal((long) mouseTime,
							savedTime, PupilViewPanel.this);
				}
				break;

			case ViewPanelListener.RESIZE_FOCUS:
			//same as set zoom by global...
			case ViewPanelListener.SETTING_ZOOM_BY_GLOBAL:
				//check if anything needs to be done
				if (!previewing)
					break;

				//turn off previewing
				previewing = false;
				previewLocalTimeByGlobal(-1, -1);

				mouseTime = globalPupilView.getTime(e.getX());
				//	cap the times by the global boundaries
				if (mouseTime < getMinGlobal())
					mouseTime = getMinGlobal();
				if (mouseTime > getMaxGlobal())
					mouseTime = getMaxGlobal();
				if (savedTime < mouseTime) {
					setLocal(savedTime, (long) Math.ceil(mouseTime));
					sessionHandler.setSelectedViewLocal(savedTime,
							(long) mouseTime, PupilViewPanel.this);
				} else {
					setLocal((long) mouseTime, savedTime);
					sessionHandler.setSelectedViewLocal((long) mouseTime,
							savedTime, PupilViewPanel.this);
				}

				break;

			case ViewPanelListener.MOVING_FOCUS:
				//check if anything needs to be done...
				if (!previewing)
					break;

				//turn off previewing
				previewing = false;
				previewLocalTimeByGlobal(-1, -1);

				//reset times
				mouseTime = globalPupilView.getTime(e.getX());
				long minPreviewFocus = (long) mouseTime - savedTime;
				long maxPreviewFocus = minPreviewFocus
						+ (getMaxLocal() - getMinLocal());
				if (minPreviewFocus < getMinGlobal()) {
					maxPreviewFocus = getMinGlobal()
							+ (maxPreviewFocus - minPreviewFocus);
					minPreviewFocus = getMinGlobal();
				}
				if (maxPreviewFocus > getMaxGlobal()) {
					minPreviewFocus = getMaxGlobal()
							- (maxPreviewFocus - minPreviewFocus);
					maxPreviewFocus = getMaxGlobal();
				}

				setLocal(minPreviewFocus, maxPreviewFocus);
				sessionHandler.setSelectedViewLocal(minPreviewFocus,
						maxPreviewFocus, PupilViewPanel.this);

				break;
			}

			//always clear out action
			action = ViewPanelListener.NOTHING;
		}
	}

	/* the viewpanel listener for this instance */
	private ViewPanelListener listener;

	/**
	 * default constructor
	 * 
	 * @param sessionHandler -
	 *            the controlling session handler
	 */
	public PupilViewPanel(SessionHandler sessionHandler) {
		super(sessionHandler);

		//initialize the data
		perspectiveData = new PupilViewData();

		//initialize the viewing rectangles
		localTaskRect = new Rectangle();
		globalTaskRect = new Rectangle();
		localPupilRect = new Rectangle();
		globalPupilRect = new Rectangle();

		//initialize the time ruler stuff
		topLocalTimeRulerRect = new Rectangle();
		botLocalTimeRulerRect = new Rectangle();
		topLocalTimeRuler = new LocalRulerDecorator(new TimeRuler(
				topLocalTimeRulerRect, false));
		botLocalTimeRuler = new LocalRulerDecorator(new TimeRuler(
				botLocalTimeRulerRect, true));

		//initialize the dashed stroke
		dashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);

		//initialize the task views
		localTaskView = new LocalWindowDecorator(dashed, new TaskView(
				localTaskRect));
		globalTaskView = new TaskView(globalTaskRect);

		//initialize the pupil views
		localPupilView = new LocalWindowDecorator(dashed, new PupilVisWindow(
				localPupilRect));
		globalPupilView = new PupilVisWindow(globalPupilRect);

		//set up the global composite
		globalComposite = new GlobalWindowDecorator(
				new GlobalVisWindowCompositor(globalTaskView, globalPupilView));

		//initialize the text layout stuff
		localText = new String("Detail");
		globalText = new String("Overview");
		f = new Font("Helvetica", Font.PLAIN, 11);
		df = new DecimalFormat("##");

		//initialize the magnification rectangle
		//listener.magRectangle=new Rectangle();

		//initialize the preview rects
		//listener.selPreviewRect=new Rectangle();
		//listener.zoomPreviewRect=new Rectangle();

		//set up the listener...
		listener = new ViewPanelListener();
		addMouseListener(listener);
		addMouseMotionListener(listener);
		setFocusable(true);

		//set animation thread running
		animationThread = new Thread(this);
		animationThread.start();

		//Register with the shared instance of TooltipManager
		ToolTipManager.sharedInstance().registerComponent(this);
		this.setBackground(Color.WHITE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Component#reshape(int, int, int, int)
	 */
	public void reshape(int x, int y, int w, int h) {
		super.reshape(x, y, w, h);

		//set the shape and location of each view

		/*
		 * - buffer space - 7 pix - ruler - 20 pix - 2 pix break - local task -
		 * buffer space - 7 pix - local pupil - 2 pix break - ruler - 20 pix -
		 * buffer space - 7 pix - buffer space - 7 pix - global task - global
		 * pupil - buffer space - 7 pix
		 */

		//the buffer heights
		int bufferHeight = 7;
		int rulerHeight = 20;
		int bufferBreak = 2;

		//local view width ~90%
		int localWidth = (int) (.9 * w);
		//local view horizontal offset
		int localOffset = (int) (.05 * w);
		//local view height ~30%
		int localHeight = (int) ((h - 5 * bufferHeight - 2 * rulerHeight - bufferBreak) * .3);

		//global view width ~80%
		int globalWidth = (int) (.8 * w);
		//global view horizontal offset
		int globalOffset = (int) (.1 * w);
		//global view height ~20%
		int globalHeight = (h - 5 * bufferHeight - 2 * rulerHeight
				- bufferBreak - 2 * localHeight) / 2;

		//top local time ruler
		topLocalTimeRulerRect.setSize(localWidth, rulerHeight);
		topLocalTimeRulerRect.setLocation(localOffset, bufferHeight);

		//local task
		localTaskRect.setSize(localWidth, localHeight);
		localTaskRect.setLocation(localOffset, (int) topLocalTimeRulerRect
				.getMaxY()
				+ bufferBreak);

		//local pupil
		localPupilRect.setSize(localWidth, localHeight);
		localPupilRect.setLocation(localOffset, (int) localTaskRect.getMaxY()
				+ bufferHeight);

		//bottom local time ruler
		botLocalTimeRulerRect.setSize(localWidth, rulerHeight);
		botLocalTimeRulerRect.setLocation(localOffset, (int) localPupilRect
				.getMaxY()
				+ bufferBreak);

		//set the ySeperator value
		ySeperator = (int) botLocalTimeRulerRect.getMaxY() + bufferHeight;

		//global task rect
		globalTaskRect.setSize(globalWidth, globalHeight);
		globalTaskRect.setLocation(globalOffset, ySeperator + bufferHeight);

		//global pupil rect
		globalPupilRect.setSize(globalWidth, globalHeight);
		globalPupilRect.setLocation(globalOffset, (int) globalTaskRect
				.getMaxY());

		//reset the magnification rectangle
		//listener.magRectangle.height=0;
		/*
		 * if(perspectiveData.getTaskBlocks()!=null)
		 * listener.magRectangle.height += globalHeight;
		 * if(perspectiveData.getPupilData()!=null) listener.magRectangle.height +=
		 * globalHeight;
		 * 
		 * listener.magRectangle.setLocation(globalOffset,globalTaskRect.y);
		 */
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		update(g);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.Component#update(java.awt.Graphics)
	 */
	public void update(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;

		//draw the local views
		localTaskView.drawWindow(g2);
		localPupilView.drawWindow(g2);

		//draw the global views
		globalComposite.drawWindow(g2);

		//draw task boundaries
		if (perspectiveData.getPupilData() != null
				&& perspectiveData.getTaskBlocks() != null) {
			drawTaskBoundaries(g2);
		}

		//draw the time rulers
		topLocalTimeRuler.drawWindow(g2);
		botLocalTimeRuler.drawWindow(g2);

		//draw box overlays of the views
		g2.setColor(Color.BLUE);
		g2.setStroke(new BasicStroke(1.0f));
		g2.draw(localTaskRect);
		g2.draw(localPupilRect);
		g2.setColor(Color.BLACK);
		g2.draw(globalTaskRect);
		g2.draw(globalPupilRect);

		//draw playhead
		drawPlayHead(g2);

		//draw the local view labels
		localPupilView.drawLabels(g2, f);
		localTaskView.drawLabels(g2, f);

		//draw the boundary selector...
		if (perspectiveData.getPupilData() != null
				&& perspectiveData.getTaskBlocks() != null) {
			boundsSelector.draw(g2, localTaskRect);
		}

		//draw line seperating 2 view sections
		g2.setColor(Color.GRAY);
		g2.setStroke(new BasicStroke(1.0f));
		g2.drawLine(0, ySeperator, getWidth(), ySeperator);

		//labels seperating local and global frames
		g2.setColor(Color.GRAY);
		FontRenderContext frc = g2.getFontRenderContext();
		TextLayout localLayout = new TextLayout(localText, f, frc);
		localLayout.draw(g2, 2, ySeperator - 2);
		TextLayout globalLayout = new TextLayout(globalText, f, frc);
		globalLayout.draw(g2, 2, (float) (ySeperator + 3 + globalLayout
				.getBounds().getHeight()));
	}

	/**
	 * method sets the pupil data for this view
	 * 
	 * @param pupilData -
	 *            the pupil packets
	 * @param baseline -
	 *            the baseline pupil size
	 * @param minPCPS -
	 *            the minimum pcps
	 * @param maxPCPS -
	 *            the maximum pcps
	 */
	public void setPupilData(Vector pupilData, float baseline, float minPCPS,
			float maxPCPS) {
		perspectiveData.setPupilData(pupilData);
		perspectiveData.setBaseline(baseline);
		perspectiveData.setExtremePCPS(minPCPS, maxPCPS);

		//set the pupil data for the pupil views
		((PupilVisWindow) ((LocalWindowDecorator) localPupilView)
				.getVisWindow()).setData(pupilData);
		((PupilVisWindow) ((LocalWindowDecorator) localPupilView)
				.getVisWindow()).setBaseLine(baseline);
		((PupilVisWindow) ((LocalWindowDecorator) localPupilView)
				.getVisWindow()).setExtremePCPS(minPCPS, maxPCPS);
		((PupilVisWindow) ((LocalWindowDecorator) localPupilView)
				.getVisWindow()).setMaskThreshold(minPCPS, true);
		((PupilVisWindow) globalPupilView).setData(pupilData);
		((PupilVisWindow) globalPupilView).setBaseLine(baseline);
		((PupilVisWindow) globalPupilView).setExtremePCPS(minPCPS, maxPCPS);
		((PupilVisWindow) globalPupilView).setMaskThreshold(minPCPS, true);

		//check if the context times need to be reset...
		if (getMinGlobal() == getMaxGlobal() && getMinGlobal() == -1) {

			long newMin = ((DataPacket) pupilData.firstElement())
					.getTimeStamp();
			long newMax = ((DataPacket) pupilData.lastElement()).getTimeStamp();

			//reset the context and the focus
			setGlobal(newMin, newMax);
			setLocal(newMin, newMax);
		}
	}

	/**
	 * method sets the task model for this view
	 * 
	 * @param taskBlocks -
	 *            the task model
	 */
	public void setTaskModel(Vector taskBlocks) {
		perspectiveData.setTaskBlocks(taskBlocks);

		//set the color if we need to....
		if (perspectiveData.getTaskLevelColors() == null
				|| perspectiveData.getTaskLevelColors().length != taskBlocks
						.size()) {
			Color[] levelColors = new Color[taskBlocks.size()];

			for (int i = 0; i < levelColors.length; i++) {
				levelColors[i] = Color.getHSBColor((float) i
						/ levelColors.length, .6f,
						((float) i / levelColors.length) * .3f + 1f
								- ((float) i / levelColors.length));
			}

			perspectiveData.setTaskLevelColors(levelColors);
		}

		//give the data to the task views...
		((TaskView) ((LocalWindowDecorator) localTaskView).getVisWindow())
				.setData(taskBlocks, perspectiveData.getTaskLevelColors());
		((TaskView) globalTaskView).setData(taskBlocks, perspectiveData
				.getTaskLevelColors());

		//check if the context times need to be reset...
		if (getMinGlobal() == getMaxGlobal() && getMinGlobal() == -1) {

			long newMin = ((TaskBlock) ((Vector) taskBlocks.get(0)).get(0))
					.getStartTime();
			long newMax = ((TaskBlock) ((Vector) taskBlocks.get(0)).get(0))
					.getEndTime();

			//reset the focus and context
			setLocal(newMin, newMax);
			setGlobal(newMin, newMax);
		}

		//construct the task boundary data structure from the task model
		taskBoundaries = new Vector(taskBlocks.size());
		for (int i = 0; i < taskBlocks.size(); i++) {
			//the level we're looking at...
			Vector thisLevel = (Vector) taskBlocks.get(i);
			//the parent level....
			Vector parentLevel = null;
			if (i != 0)
				parentLevel = (Vector) taskBlocks.get(i - 1);

			//add new taskboundary vector
			taskBoundaries.add(i, new Vector());
			for (int block = 0; block < thisLevel.size(); block++) {
				long thisstart = ((TaskBlock) thisLevel.get(block))
						.getStartTime();
				long thisend = ((TaskBlock) thisLevel.get(block)).getEndTime();

				//if this is the highest level, just copy the boundaries
				//from these task blocks
				if (i == 0) {
					((Vector) this.taskBoundaries.get(i)).add(new Long(
							thisstart));
					((Vector) this.taskBoundaries.get(i))
							.add(new Long(thisend));
				} else {
					//otherwise, make sure these boundaries are unique for this
					// level
					boolean startok = true;
					boolean endok = true;

					for (int parentblock = 0; parentblock < parentLevel.size(); parentblock++) {
						long parentstart = ((TaskBlock) parentLevel
								.get(parentblock)).getStartTime();
						long parentend = ((TaskBlock) parentLevel
								.get(parentblock)).getEndTime();

						if (parentstart == thisstart || thisstart == parentend) {
							startok = false;
						}
						if (parentend == thisend || thisend == parentstart) {
							endok = false;
						}

						if (!endok && !startok)
							break;
					}

					if (startok)
						((Vector) this.taskBoundaries.get(i)).add(new Long(
								thisstart));
					if (endok)
						((Vector) this.taskBoundaries.get(i)).add(new Long(
								thisend));
				}
			}

		}

		//initialize drawlevels
		drawLevels = new boolean[taskBlocks.size()];
		for (int i = 0; i < drawLevels.length; i++)
			drawLevels[i] = false;

		//initialize the bounds selector...
		boundsSelector = new BoundarySelector(perspectiveData
				.getTaskLevelColors());
	}

	/**
	 * method sets the context for this view
	 * 
	 * @param minTime -
	 *            the min context time
	 * @param maxTime -
	 *            the max context time
	 */
	public void setGlobal(long minTime, long maxTime) {
		//update view data, global views
		super.setGlobal(minTime, maxTime);

		globalPupilView.setViewTimes(minTime, maxTime);
		globalTaskView.setViewTimes(minTime, maxTime);
		repaint();
	}

	/**
	 * method sets the focus for this view
	 * 
	 * @param minTime -
	 *            min focus time
	 * @param maxTime -
	 *            max focus time
	 */
	public void setLocal(long minTime, long maxTime) {
		//clear out the selection data if necessary...
		if (getMinSelection() != getMaxSelection()) {
			if (getMinSelection() < minTime || getMinSelection() > maxTime
					|| getMaxSelection() < minTime
					|| getMaxSelection() > maxTime) {
				sessionHandler.setSelectedViewSelection((long) -1, (long) -1,
						(view.View) this);
				setSelection(-1, -1);
			}
		}

		//update view data
		super.setLocal(minTime, maxTime);
		localPupilView.setViewTimes(minTime, maxTime);
		localTaskView.setViewTimes(minTime, maxTime);
		globalComposite.setLocalTimes(minTime, maxTime);
		topLocalTimeRuler.setViewTimes(minTime, maxTime);
		botLocalTimeRuler.setViewTimes(minTime, maxTime);

		repaint();
	}

	/**
	 * method sets the selection for this view
	 * 
	 * @param minTime -
	 *            min selection time
	 * @param maxTime -
	 *            max selection time
	 */
	public void setSelection(long minTime, long maxTime) {
		//update view data
		super.setSelection(minTime, maxTime);

		localPupilView.setSelectionTimes(minTime, maxTime);
		localTaskView.setSelectionTimes(minTime, maxTime);
		globalComposite.setSelectionTimes(minTime, maxTime);
		topLocalTimeRuler.setSelectionTimes(minTime, maxTime);
		botLocalTimeRuler.setSelectionTimes(minTime, maxTime);

		repaint();
	}

	/**
	 * method returns settings data concerning this view. settings are returned
	 * in the form of a vector. the order of the vector is as follows:
	 * (String)view name, (Long)min local time, (Long) max local time, (Long)min
	 * selection time, (Long)max selection time, (Long) min context time, (Long)
	 * max context time, (String)notes,(String) context menu item
	 * 
	 * @return Vector representing view settings
	 */
	public Vector exportSettings() {
		Vector settings = new Vector();
		settings.add(getName());
		settings.add(new Long(getMinLocal()));
		settings.add(new Long(getMaxLocal()));
		settings.add(new Long(getMinSelection()));
		settings.add(new Long(getMaxSelection()));
		settings.add(new Long(getMinGlobal()));
		settings.add(new Long(getMaxGlobal()));
		settings.add(getNotes());
		settings.add(getGlobalMenuText());

		return settings;
	}

	/**
	 * method sets settings data in the same form listed in export settings
	 * 
	 * @param settings -
	 *            see exportSettings for vector format
	 */
	public void importSettings(Vector settings) {
		//set the name
		this.setName((String) settings.get(0));
		this.setGlobal(((Long) settings.get(5)).longValue(), ((Long) settings
				.get(6)).longValue());
		this.setLocal(((Long) settings.get(1)).longValue(), ((Long) settings
				.get(2)).longValue());
		this.setSelection(((Long) settings.get(3)).longValue(),
				((Long) settings.get(4)).longValue());

		//set the notes
		this.setNotes((String) settings.get(7));

		//set the context menu name
		setGlobalMenuText((String) settings.get(8));

	}

	/**
	 * method previews a local zoom via a local action
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	private void previewLocalTimeByLocal(long minTime, long maxTime) {
		localPupilView.previewLocalTimesByLocal(minTime, maxTime);
		localTaskView.previewLocalTimesByLocal(minTime, maxTime);
		globalComposite.previewLocalTimesByLocal(minTime, maxTime);
		topLocalTimeRuler.previewLocalTimesByLocal(minTime, maxTime);
		botLocalTimeRuler.previewLocalTimesByLocal(minTime, maxTime);
	}

	/**
	 * method previews a local zoom via a global interaction
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	private void previewLocalTimeByGlobal(long minTime, long maxTime) {
		localPupilView.previewLocalTimesByGlobal(minTime, maxTime);
		localTaskView.previewLocalTimesByGlobal(minTime, maxTime);
		globalComposite.previewLocalTimesByGlobal(minTime, maxTime);
		topLocalTimeRuler.previewLocalTimesByGlobal(minTime, maxTime);
		botLocalTimeRuler.previewLocalTimesByGlobal(minTime, maxTime);
	}

	/**
	 * method previews a selection
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewSelection(long minTime, long maxTime) {
		localPupilView.previewSelectionTimes(minTime, maxTime);
		localTaskView.previewSelectionTimes(minTime, maxTime);
		globalComposite.previewSelectionTimes(minTime, maxTime);
		topLocalTimeRuler.previewSelectionTimes(minTime, maxTime);
		botLocalTimeRuler.previewSelectionTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		long time = System.currentTimeMillis();
		while (true) {//infinite loop

			try {
				time += animationDelay;

				if (listener.previewing
						&& (listener.action == ViewPanelListener.SETTING_SEL_BY_LOCAL_PUPIL
								|| listener.action == ViewPanelListener.SETTING_SEL_BY_LOCAL_TASK
								|| listener.action == ViewPanelListener.SETTING_ZOOM_BY_LOCAL_PUPIL || listener.action == ViewPanelListener.SETTING_ZOOM_BY_LOCAL_TASK)) {

					dashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT,
							BasicStroke.JOIN_MITER, 10.0f, dash1,
							((time) / 100) % (int) (dash1[0] * 2));

					((LocalWindowDecorator) localPupilView)
							.setPreviewStroke(dashed);
					((LocalWindowDecorator) localTaskView)
							.setPreviewStroke(dashed);

					//should we repaint?
					if (perspectiveData.getPupilData() != null
							|| perspectiveData.getTaskBlocks() != null)
						repaint();
				}

				Thread.sleep(Math.max(0, time - System.currentTimeMillis()));
			} catch (InterruptedException e) {
				System.out.println(e);
			}
		}
	}

	/**
	 * @return the tooltip
	 */
	public String getToolTipText(java.awt.event.MouseEvent e) {

		//local pupil view?
		if (perspectiveData.getPupilData() != null
				&& localPupilRect.contains(e.getPoint())) {
			return localPupilView.getToolTip(e);
		}

		//local task model?
		if (perspectiveData.getTaskBlocks() != null
				&& localTaskRect.contains(e.getPoint())) {
			return localTaskView.getToolTip(e);
		}

		//nothing...
		return null;
	}

	/**
	 * the boundary selector widget.
	 * 
	 * @author chris
	 *  
	 */
	private class BoundarySelector {

		/**
		 * Boundary Widgets are used for capturing mouse clicks
		 * 
		 * @author chris
		 *  
		 */
		private class BoundWidget extends Rectangle {

			/** the color of the widget* */
			private Color theColor;

			/** flag indicating whether this widget is selected* */
			private boolean selected;

			/**
			 * default constructor
			 * 
			 * @param aColor
			 */
			public BoundWidget(Color aColor) {
				this.theColor = aColor;
				this.selected = false;
			}

			/**
			 * method draws the widget...
			 * 
			 * @param g2
			 */
			public void draw(Graphics2D g2) {
				//draw fill rect
				g2.setColor(this.theColor);
				g2.fill(this);

				//draw outline
				g2.setColor(Color.black);
				g2.setStroke(new BasicStroke(1.0f));
				g2.draw(this);

				//are we selected?
				if (selected) {
					g2.drawLine((int) this.getMinX(), (int) this.getMinY(),
							(int) this.getMaxX(), (int) this.getMaxY());
					g2.drawLine((int) this.getMaxX(), (int) this.getMinY(),
							(int) this.getMinX(), (int) this.getMaxY());
				}
			}

			/**
			 * @return Returns the selected.
			 */
			public boolean isSelected() {
				return selected;
			}

			/**
			 * @param selected
			 *            The selected to set.
			 */
			public void setSelected(boolean selected) {
				this.selected = selected;
			}

			/**
			 * @return Returns the theColor.
			 */
			public Color getTheColor() {
				return theColor;
			}

			/**
			 * @param theColor
			 *            The theColor to set.
			 */
			public void setTheColor(Color theColor) {
				this.theColor = theColor;
			}

		}

		/** the boundary widgets for this selector* */
		private BoundWidget[] widgets;

		/** the index of the currently selected widget* */
		private int selWidget;

		/**
		 * default constructor
		 * 
		 * @param levelColors
		 */
		public BoundarySelector(Color[] levelColors) {
			//initialize the widgets, set myself as a listener
			widgets = new BoundWidget[levelColors.length];
			for (int i = 0; i < widgets.length; i++) {
				widgets[i] = new BoundWidget(levelColors[i]);
			}
			selWidget = 0;
		}

		/**
		 * method checks which level the mouse is at
		 * 
		 * @param e
		 * @return -1 if no hit, otherwise return level
		 */
		public int checkIntersect(MouseEvent e) {
			int levelClicked = -1;

			//check if a widget was clicked
			for (int i = 0; i < widgets.length; i++) {
				if (widgets[i].contains(e.getX(), e.getY())) {
					//bw.setSelected(!bw.isSelected());
					levelClicked = i;
					selWidget = i;
					break;
				}
			}

			return levelClicked;
		}

		/**
		 * method draws the selector in reference to the view rect
		 * 
		 * @param g2
		 * @param viewRect
		 */
		public void draw(Graphics2D g2, Rectangle viewRect) {
			//the height of each task block
			int height = (int) (viewRect.getHeight() / widgets.length);
			//the remainder pixels
			int remainder = (int) viewRect.getHeight() - height
					* widgets.length;

			//the top pixel of the level
			int top = (int) viewRect.getMinY();
			//the bottom pixel of the level
			int bottom = 0;
			for (int level = 0; level < widgets.length; level++) {
				//set the bottom pixel
				if (remainder > 0) {
					bottom = top + height + 1;
					remainder--;
				} else
					bottom = top + height;

				widgets[level].setBounds(5 + (int) viewRect.getMaxX(), top,
						(int) (height / 1.65), bottom - top);
				widgets[level].draw(g2);

				top = bottom;
			}

			//also draw the bracket
			g2.setColor(Color.BLACK);
			g2.setStroke(new BasicStroke(3));
			g2.drawLine((int) widgets[0].getMaxX(), (int) widgets[0].getMinY(),
					(int) widgets[0].getMaxX() + 7, (int) widgets[0].getMinY());
			g2.drawLine((int) widgets[0].getMaxX() + 7, (int) widgets[0]
					.getMinY(), (int) widgets[selWidget].getMaxX() + 7,
					(int) widgets[selWidget].getMaxY());
			g2.drawLine((int) widgets[selWidget].getMaxX(),
					(int) widgets[selWidget].getMaxY(),
					(int) widgets[selWidget].getMaxX() + 7,
					(int) widgets[selWidget].getMaxY());
		}
	}

	/**
	 * method draws the task boundaries over the local pupil view
	 * 
	 * @param g2
	 */
	private void drawTaskBoundaries(Graphics2D g2) {
		if (taskBoundaries == null)
			return;

		for (int level = 0; level < taskBoundaries.size(); level++) {
			//are we even drawing this level?
			if (!drawLevels[level])
				continue;

			Vector levelBlocks = (Vector) taskBoundaries.get(level);

			for (int i = 0; i < levelBlocks.size(); i++) {
				//TaskBlock tb = (TaskBlock)levelBlocks.get(i);
				long time = ((Long) levelBlocks.get(i)).longValue();

				g2.setColor(perspectiveData.getTaskLevelColors()[level
						% perspectiveData.getTaskLevelColors().length]);
				g2.setStroke(new BasicStroke((float) 1.0));

				//only draw if we care
				if (time >= getMinLocal() && time <= getMaxLocal()) {
					int timeX = (int) localPupilView.getXCoord(time);
					g2.drawLine(timeX, (int) localPupilRect.getMinY(), timeX,
							(int) localPupilRect.getMaxY());
				}
			}
		}
	}

	/**
	 * mehthod toggles the given level boundaries to be drawn
	 * 
	 * @param level
	 * @param isSet
	 */
	public void toggleBoundary(int level, boolean isSet) {
		if (drawLevels == null || level < 0 || level > drawLevels.length)
			return;

		drawLevels[level] = isSet;
	}

	/**
	 * @return Returns the playHead time.
	 */
	public double getPlayHead() {
		return playHead;
	}

	/**
	 * @param playHead
	 *            The playHead to set.
	 */
	public void setPlayHead(double playHead) {
		this.playHead = playHead;
	}

	/**
	 * method draws the playhead
	 * 
	 * @param g2
	 */
	private void drawPlayHead(Graphics2D g2) {
		if (playHead == -1)
			return;

		//the playhead position in x coords
		int localX = (int) localPupilView.getXCoord(playHead);
		int globalX = (int) globalPupilView.getXCoord(playHead);

		//cap the playhead position
		if (localX < localTaskRect.x)
			localX = localTaskRect.x;
		if (localX > localTaskRect.getMaxX())
			localX = (int) localTaskRect.getMaxX();
		if (globalX < globalTaskRect.x)
			globalX = globalTaskRect.x;
		if (globalX > globalTaskRect.getMaxX())
			globalX = (int) globalTaskRect.getMaxX();

		g2.setColor(Color.red);
		g2.setStroke(new BasicStroke(1.0f));

		if (perspectiveData.getPupilData() != null) {
			g2.drawLine(localX, (int) localPupilRect.getMinY(), localX,
					(int) localPupilRect.getMaxY());
			g2.drawLine(globalX, (int) globalPupilRect.getMinY(), globalX,
					(int) globalPupilRect.getMaxY());
		}

		if (perspectiveData.getTaskBlocks() != null) {
			g2.drawLine(localX, (int) localTaskRect.getMinY(), localX,
					(int) localTaskRect.getMaxY());
			g2.drawLine(globalX, (int) globalTaskRect.getMinY(), globalX,
					(int) globalTaskRect.getMaxY());
		}

		//draw the play head on the time rulers
		g2.drawLine(localX, (int) topLocalTimeRulerRect.getMinY(), localX,
				(int) topLocalTimeRulerRect.getMaxY());
		g2.drawLine(localX, (int) botLocalTimeRulerRect.getMinY(), localX,
				(int) botLocalTimeRulerRect.getMaxY());
	}

	/**
	 * method sets the masking threshold for the pupil views
	 * 
	 * @param threshold
	 */
	public void setPupilMask(float threshold, boolean willReScaleOnMask) {
		((PupilVisWindow) ((LocalWindowDecorator) localPupilView)
				.getVisWindow()).setMaskThreshold(threshold, willReScaleOnMask);
		((PupilVisWindow) globalPupilView).setMaskThreshold(threshold,
				willReScaleOnMask);
		((PupilVisWindow) ((LocalWindowDecorator) localPupilView)
				.getVisWindow()).forceRedraw();
		((PupilVisWindow) globalPupilView).forceRedraw();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.View#purge()
	 */
	public void purge() {
		setLocal(-1, -1);
		setGlobal(-1, -1);
		setSelection(-1, -1);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.View#previewLocal(long, long)
	 */
	public void previewLocal(long minTime, long maxTime) {
		//figure out what would be the best preview to do...
		if (minTime >= getMinLocal() && maxTime <= getMaxLocal())
			previewLocalTimeByLocal(minTime, maxTime);
		else
			previewLocalTimeByGlobal(minTime, maxTime);

	}
}