/*
 * Created on Jul 6, 2004
 *
 */
package task;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * @author chris
 *  
 */
public class ReadTaskXML {
	/** the file to read in */
	private String file;

	/** document read in by the xml parser* */
	static Document document;

	/** taskBlocks is a 2 dim Vector of task blocks */
	private Vector taskBlocks;

	/** the list of elements we're interested in */
	static String[] elementNames = { "task", "start", "end", };

	/**
	 * 
	 * @param file -
	 *            the file to read in
	 */
	public ReadTaskXML(String file) {
		this.file = file;
		taskBlocks = new Vector();
	}

	/**
	 * used when parsing the xml file
	 * 
	 * @param theNode -
	 *            the node we're looking at
	 * @param level -
	 *            the current level
	 * @param parent -
	 *            this nodes parent
	 * @param offset -
	 *            the accumulated offset for this node
	 */
	public void assignTaskBlock(Node theNode, int level, TaskBlock parent,
			long offset) {
		if (theNode == null)
			return;

		//the potential taskblock for this node
		TaskBlock myBlock = null;

		//check which type of node we're looking at...
		//is it an element?....
		if (theNode.getNodeType() == 1) {

			//what type of element?
			//task...
			if (theNode.getNodeName().equals(elementNames[0])) {

				//since this is a task, create a task block...
				myBlock = new TaskBlock();

				//set the name...
				if (theNode.getAttributes().getNamedItem("name") != null)
					myBlock.setName((String) theNode.getAttributes()
							.getNamedItem("name").getNodeValue());

				//is there an offset?
				if (theNode.getAttributes().getNamedItem("offset") != null) {
					offset += Long.parseLong(theNode.getAttributes()
							.getNamedItem("offset").getNodeValue());
				}

				//set the level...
				myBlock.setLevel(level);

				//if we can, tell the parent we are it's child
				if (parent != null)
					parent.getTaskChildren().add(myBlock);

				//add the block to the task Blocks data structure
				if (taskBlocks.size() < level) {
					taskBlocks.add(new Vector());
				}

				((Vector) taskBlocks.get(level - 1)).add(myBlock);

				//System.out.println(myBlock.getName());
			} else
			//start...
			if (theNode.getNodeName().equals(elementNames[1])) {
				//set the parent start time...
				if (parent != null) {
					if (theNode.getChildNodes().getLength() != 1)
						return;
					parent.setStartTime(Long.parseLong(theNode.getChildNodes()
							.item(0).getNodeValue())
							+ offset);
				}

				//we don't need to recurse anymore
				return;

			} else
			//end...
			if (theNode.getNodeName().equals(elementNames[2])) {
				//set the parent end time...
				if (parent != null) {
					if (theNode.getChildNodes().getLength() != 1)
						return;
					parent.setEndTime(Long.parseLong(theNode.getChildNodes()
							.item(0).getNodeValue())
							+ offset);
				}

				//we don't need to recurse anymore
				return;
			}
		}

		//recurse...
		for (int i = 0; i < theNode.getChildNodes().getLength(); i++) {
			assignTaskBlock(theNode.getChildNodes().item(i), level + 1,
					myBlock, offset);
		}

		//if the node is a task, set the start and end times based on
		//the children...
		if (myBlock != null && myBlock.getTaskChildren().size() != 0) {
			myBlock.setStartTime(((TaskBlock) myBlock.getTaskChildren()
					.firstElement()).getStartTime());
			myBlock.setEndTime(((TaskBlock) myBlock.getTaskChildren()
					.lastElement()).getEndTime());
		}
	}

	/**
	 * method actually reads the xml document
	 * 
	 * @throws SAXException
	 * @throws IOException
	 * @throws ParserConfigurationException
	 */
	public void read() throws SAXException, IOException,
			ParserConfigurationException {

		//parse the file...
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		DocumentBuilder builder = factory.newDocumentBuilder();
		document = builder.parse(new File(file));

		//get the document root
		Node rootNode = document;

		//set the taskBlocks datastructure....
		//start recursing on the document's root node, level 0,
		//with no parent
		assignTaskBlock(rootNode, 0, null, 0);

	}

	/**
	 * 
	 * @return the data
	 */
	public Vector getData() {
		return taskBlocks;
	}
}