/*
 * Created on Aug 24, 2004
 * VideoModule.java
 * 
 * Project: eyetrack_cvs
 */
package video;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import pupil.PupilSessionHandler;
import view.View;

//import quicktime.QTException;
//import quicktime.app.view.QTFactory;
//import quicktime.io.OpenMovieFile;
//import quicktime.io.QTFile;
//import quicktime.std.StdQTException;
//import quicktime.std.clocks.TimeRecord;
//import quicktime.std.movies.Movie;
//import quicktime.std.movies.MovieController;
//import quicktime.std.movies.MovieDrawingComplete;

/**
 * @author chris
 *  
 */
public class VideoView extends JDialog implements View {
	/** default movie size */
	public static int MOVIE_WIDTH = 400, MOVIE_HEIGHT = 400;

	/** "play" controller button */
	protected JButton playBu;

	/** "rewind" controller button */
	protected JButton rewBu;

	/** "fast forward" controller button */
	protected JButton ffBu;

	/** the movie object... */
	protected MYMovie movie;

	/** offset used for syncing video with sim clock */
	protected long offset;

	/** the current min focus */
	protected long minFocus;

	/** the current max focus */
	protected long maxFocus;

	/** the current min context */
	protected long minContext;

	/** the current max context */
	protected long maxContext;

	/** the current min selection */
	protected long minSelect;

	/** the current max selection */
	protected long maxSelect;

	/** our video content goes here */
	protected Component canvas;

	/** the movie controller */
	protected MYMovieController controller;

	/** the min tick of the playback */
	protected int minTick;

	/** the max tick of the playback */
	protected int maxTick;

	/** the time slider* */
	protected JSlider timeSlider;

	/** the current video time label */
	protected JLabel currentTimeLabel;

	/** the movie bounds time label */
	protected JLabel timeBoundLabel;

	/** flag indicates the timeslider is being moved by the user* */
	protected boolean adjustSlider = false;

	/** the session handler for this video* */
	private PupilSessionHandler sessionHandler;

	/** buttonlistener class* */
	private class ButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (movie == null)
				return;

			if (e.getSource() == playBu) {
				try {
					if (movie.getRate() == 0) {
						movie.setRate(1);
					} else {
						movie.setRate(0);
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			} else if (e.getSource() == rewBu) {
				try {
					//try to back up in time...
					int newTick = getTickFromTime(getTimeFromTick(movie
							.getTime()) - 2000);
					if (newTick < minTick)
						movie.setTimeValue(minTick);
					else
						movie.setTimeValue(newTick);
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			} else if (e.getSource() == ffBu) {
				try {
					//try to forward in time...
					int newTick = getTickFromTime(getTimeFromTick(movie
							.getTime()) + 2000);
					if (newTick > maxTick)
						movie.setTimeValue(maxTick);
					else
						movie.setTimeValue(newTick);
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}

		}

	}

	/**
	 * default constructor
	 * 
	 * @param owner -
	 *            the parent jframe
	 * @param sessionHandler
	 * @throws QTException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 */
	public VideoView(JFrame owner, PupilSessionHandler sessionHandler)
			throws /* QTException, */SecurityException,
			IllegalArgumentException, NoSuchMethodException,
			ClassNotFoundException, IllegalAccessException,
			InvocationTargetException {
		super(owner);
		this.sessionHandler = sessionHandler;

		//we aren't modal
		setModal(false);

		//initialize the controller buttons
		playBu = new JButton("Play/Pause");
		playBu.setEnabled(false);
		rewBu = new JButton("<<");
		rewBu.setEnabled(false);
		ffBu = new JButton(">>");
		ffBu.setEnabled(false);

		//initialize the timeSlider
		timeSlider = new JSlider(JSlider.HORIZONTAL, minTick, maxTick, minTick);
		timeSlider.setEnabled(false);

		//add change/mouse listener to the slider
		timeSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent arg0) {
				//set the time label
				try {
					currentTimeLabel.setText(Integer
							.toString((int) (getTimeFromTick(timeSlider
									.getValue()) / 1000))
							+ " s");
					VideoView.this.sessionHandler
							.setPlayHead(getTimeFromTick(timeSlider.getValue()));
				} catch (Exception e) {

				}

				//only care if user is manually doing something
				if (!adjustSlider)
					return;
				try {
					//update the movie as to the sliders position
					movie.setTimeValue(timeSlider.getValue());
				} catch (Exception e) {
					e.printStackTrace();
				}

			}

		});
		timeSlider.addMouseListener(new MouseListener() {

			//keeps the last stored rate before moving the slider
			float oldRate;

			public void mouseClicked(MouseEvent arg0) {
			}

			public void mouseEntered(MouseEvent arg0) {
			}

			public void mouseExited(MouseEvent arg0) {
			}

			public void mousePressed(MouseEvent arg0) {
				//set the adjust slider flag
				adjustSlider = true;

				//pause the movie
				try {
					oldRate = movie.getRate();
					movie.setRate(0.0f);
				} catch (Exception ex) {
					//what could go wrong?
				}
			}

			public void mouseReleased(MouseEvent arg0) {
				//we're done adjusting
				adjustSlider = false;

				//unpause the movie?
				try {
					movie.setRate(oldRate);
				} catch (Exception ex) {
					//what could go wrong?
				}
			}
		});

		canvas = (Component) MYQTFactory.makeQTComponent((MYMovie) null);

		//the south panel holds the video control content
		JPanel southPanel = new JPanel();
		southPanel.setLayout(new GridLayout(2, 1));
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(rewBu);
		buttonPanel.add(playBu);
		buttonPanel.add(ffBu);

		JPanel sliderPanel = new JPanel();
		currentTimeLabel = new JLabel();
		timeBoundLabel = new JLabel();
		sliderPanel.add(currentTimeLabel);
		sliderPanel.add(timeSlider);
		sliderPanel.add(timeBoundLabel);

		southPanel.add(sliderPanel);
		southPanel.add(buttonPanel);

		this.getContentPane().add(canvas, "Center");
		this.getContentPane().add(southPanel, "South");

		//add listener to the buttons...
		ButtonListener bl = new ButtonListener();
		playBu.addActionListener(bl);
		rewBu.addActionListener(bl);
		ffBu.addActionListener(bl);
		pack();
	}

	/**
	 * method sets the video to be displayed in this frame. if null is passed as
	 * the videopath, the frame is purged of movie content
	 * 
	 * @param theVideoPath -
	 *            the absolute path of the video to play
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 * @throws NoSuchMethodException
	 * @throws ClassNotFoundException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws InstantiationException
	 * @throws QTException
	 */
	public void setVideo(String theVideoPath) throws SecurityException,
			IllegalArgumentException, NoSuchMethodException,
			ClassNotFoundException, IllegalAccessException,
			InvocationTargetException, InstantiationException {

		//remove any existing callbacks...
		if (movie != null) {
			try {
				movie.removeDrawingCompleteProc();
			} catch (Exception e) {
			}
		}

		//try to set up the movie
		if (theVideoPath == null) {
			//try {
			//no video to add, so just create a blank canvas
			canvas = (Component) MYQTFactory.makeQTComponent((MYMovie) null);
			//} catch (QTException e) {
			//e.printStackTrace();
			//}

			//turn off the widgets
			playBu.setEnabled(false);
			rewBu.setEnabled(false);
			ffBu.setEnabled(false);
			timeSlider.setEnabled(false);

		} else {
			File f = new File(theVideoPath);
			MYOpenMovieFile movieFile;

			movieFile = MYOpenMovieFile.asRead(new MYQTFile(f));
			movie = null;
			movie = MYMovie.fromFile(movieFile);

			movie.getTimeBase().setFlags(MYStdQTConstants.loopTimeBase);

			//set up QTCallback...
			Class MYQTCallBackClass = ClassLoader.getSystemClassLoader()
					.loadClass("video.MYQTCallBack");
			MYQTCallBackClass.getConstructor(
					new Class[] { this.getClass(), Object.class }).newInstance(
					new Object[] { this, movie.getMovie() });

			movie.setRate(0);
			minTick = 0;
			maxTick = movie.getDuration();
			//add the movie to the canvas
			canvas = null;
			canvas = (Component) MYQTFactory.makeQTComponent(movie);

			//turn on the widgets
			playBu.setEnabled(true);
			ffBu.setEnabled(true);
			rewBu.setEnabled(true);
			timeSlider.setEnabled(true);
		}

		//replace the canvas, repaint this window
		this.getContentPane().add(canvas, "Center");
		this.pack();

		//finally, set the title for this video
		super.setTitle(theVideoPath);
	}

	/**
	 * method sets the local frame
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void setLocal(long minTime, long maxTime) {
		//set the min and max focus...
		minFocus = minTime;
		maxFocus = maxTime;

		if (movie == null)
			return;

		//check if these times are even valid....
		try {
			playBu.setEnabled(true);
			ffBu.setEnabled(true);
			rewBu.setEnabled(true);
			timeSlider.setEnabled(true);

			movie.setRate(0.0f);

			if (minFocus == maxFocus && minFocus == -1) {
				minFocus = (long) offset;
				maxFocus = (long) getTimeFromTick(movie.getDuration());
			} else if (minFocus > getTimeFromTick(movie.getDuration())
					|| maxFocus < offset) {
				//can't do anthing...
				playBu.setEnabled(false);
				ffBu.setEnabled(false);
				rewBu.setEnabled(false);
				timeSlider.setEnabled(false);
			} else {
				if (minFocus < offset)
					minFocus = (long) offset;
				if (maxFocus > getTimeFromTick(movie.getDuration()))
					maxFocus = (long) getTimeFromTick(movie.getDuration());
			}

			//set the time boundary label
			timeBoundLabel.setText("(" + minFocus / 1000 + ":" + maxFocus
					/ 1000 + ")");

			//save the old tick values, get the new ones
			int oldMinTick = minTick;
			int oldMaxTick = maxTick;
			minTick = getTickFromTime(minFocus);
			maxTick = getTickFromTime(maxFocus);

			timeSlider.setMinimum(minTick);
			timeSlider.setMaximum(maxTick);
			timeSlider.setValue(minTick);

			if (movie != null) {
				if (minTick >= 0 && minTick < movie.getDuration()) {
					//set the start and stop times in the right order to avoid
					// crashing...
					if (oldMaxTick < minTick) {
						movie.getTimeBase()
								.setStopTime(
										new MYTimeRecord(movie.getTimeScale(),
												maxTick));
						movie.getTimeBase()
								.setStartTime(
										new MYTimeRecord(movie.getTimeScale(),
												minTick));
					} else {
						movie.getTimeBase()
								.setStartTime(
										new MYTimeRecord(movie.getTimeScale(),
												minTick));
						movie.getTimeBase()
								.setStopTime(
										new MYTimeRecord(movie.getTimeScale(),
												maxTick));
					}

					//reset the movie to the min tick
					movie.setTimeValue(minTick);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * method returns the tick corresponding to some simulated time value
	 * 
	 * @param simTime -
	 *            the time to get the tick from
	 * @return tick value
	 * @throws StdQTException
	 * @throws NoSuchMethodException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 */
	private int getTickFromTime(double simTime)
			throws /* StdQTException, */IllegalArgumentException,
			SecurityException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {
		int timeValue = (int) (((simTime - offset) / 1000) * movie
				.getTimeScale());
		return timeValue;
	}

	/**
	 * method returns the simulated time corresponding to some tick
	 * 
	 * @param tick -
	 *            the tick to get the time from
	 * @return @throws
	 *         StdQTException
	 * @throws NoSuchMethodException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 */
	private double getTimeFromTick(int tick)
			throws /* StdQTException, */IllegalArgumentException,
			SecurityException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {
		double timeValue = offset
				+ ((double) tick / (double) movie.getTimeScale()) * 1000;
		return timeValue;
	}

	/**
	 * method sets the selection frame
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void setSelection(long minTime, long maxTime) {
		minSelect = minTime;
		maxSelect = maxTime;
	}

	/**
	 * method sets the context frame
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void setGlobal(long minTime, long maxTime) {
		minContext = minTime;
		maxContext = maxTime;
	}

	/**
	 * @return Returns the offset.
	 */
	public long getOffset() {
		return offset;
	}

	/**
	 * @param offset
	 *            The offset to set.
	 */
	public void setOffset(long offset) {
		this.offset = offset;
	}

	/**
	 * method called by the MYQTCallback object associated with my Movie object
	 */
	public void execute() {
		//don't do anything if we're modifying the slider
		if (adjustSlider)
			return;

		try {
			//this shouldn't really happen...
			if (movie.getTime() > maxTick) {
				movie.setRate(0);
			}

			timeSlider.setValue(movie.getTime());
			timeSlider.repaint();
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	/**
	 * preview the given focus frame
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewLocal(long minTime, long maxTime) {

	}

	/**
	 * preview the given selection frame
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewSelection(long minTime, long maxTime) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.View#purge()
	 */
	public void purge() {
		try {
			setVideo(null);
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		}

	}
}