/*
 * Created on Oct 31, 2004
 * LocalRulerDecorator.java
 * 
 * Project: eyetrack_cvs
 */
package view;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;

/**
 * @author chris
 *  
 */
public class LocalRulerDecorator extends VisWindowDecorator {

	/** the min preview selection time* */
	private long minPreviewSel = -1;

	/** the max preview selection time* */
	private long maxPreviewSel = -1;

	/** the min local preview time by local action* */
	private long minPreviewLocalByLocal = -1;

	/** the max local preview time by local action* */
	private long maxPreviewLocalByLocal = -1;

	/** the min local preview time by global action */
	private long minPreviewLocalByGlobal = -1;

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewLocalTimesByGlobal(long, long)
	 */
	public void previewLocalTimesByGlobal(long minTime, long maxTime) {
		super.previewLocalTimesByGlobal(minTime, maxTime);
		minPreviewLocalByGlobal = minTime;
		maxPreviewLocalByGlobal = maxTime;
	}

	/** the max local preview time by global action */
	private long maxPreviewLocalByGlobal = -1;

	/** the min selection time* */
	private long minSelectionTime;

	/** the max selection time* */
	private long maxSelectionTime;

	public LocalRulerDecorator(VisWindow theWindow) {
		super(theWindow);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#drawWindow(java.awt.Graphics2D)
	 */
	public void drawWindow(Graphics2D g2) {
		theWindow.drawWindow(g2);

		//are we presently displaying a selection?
		if (!((minSelectionTime == maxSelectionTime) && minSelectionTime == -1)) {
			g2.setColor(Color.LIGHT_GRAY);
			Rectangle selRect = new Rectangle();

			//draw the selection slightly lighter if we are currently
			//setting a selection
			float alpha = .35f;
			if (!((minPreviewSel == maxPreviewSel) && minPreviewSel == -1))
				alpha = .15f;

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					alpha));

			if (theWindow.getXCoord(minSelectionTime) < theWindow
					.getDrawRegion().x)
				selRect.setLocation(theWindow.getDrawRegion().x, theWindow
						.getDrawRegion().y - 1);
			else
				selRect.setLocation(theWindow.getXCoord(minSelectionTime),
						theWindow.getDrawRegion().y - 1);

			if (theWindow.getXCoord(maxSelectionTime) > theWindow
					.getDrawRegion().getMaxX())
				selRect.setSize((int) theWindow.getDrawRegion().getMaxX()
						- (int) selRect.getX(),
						theWindow.getDrawRegion().height + 2);
			else
				selRect.setSize(theWindow.getXCoord(maxSelectionTime)
						- (int) selRect.getX(),
						theWindow.getDrawRegion().height + 2);

			//draw onto the screen
			g2.fill(selRect);

			//reset the composite
			g2.setComposite(oldAC);
		}

		//are we previewing a selection?
		if (!((minPreviewSel == maxPreviewSel) && minPreviewSel == -1)) {
			g2.setColor(Color.LIGHT_GRAY);
			g2.setStroke(new BasicStroke(1.0f));

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			AlphaComposite newAC = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, (float) .35);
			g2.setComposite(newAC);

			//draw preview rectangle...
			Rectangle selPreviewRect = new Rectangle();
			selPreviewRect.setLocation(theWindow.getXCoord(minPreviewSel),
					theWindow.getDrawRegion().y - 1);
			selPreviewRect.setSize(theWindow.getXCoord(maxPreviewSel)
					- (int) selPreviewRect.getX(),
					theWindow.getDrawRegion().height + 2);
			g2.fill(selPreviewRect);

			g2.setComposite(oldAC);

			g2.drawLine(selPreviewRect.x, selPreviewRect.y, selPreviewRect.x,
					(int) selPreviewRect.getMaxY());
			g2.drawLine((int) selPreviewRect.getMaxX(), selPreviewRect.y,
					(int) selPreviewRect.getMaxX(), (int) selPreviewRect
							.getMaxY());
		}

		//are we previewing a local zoom?
		if (!((minPreviewLocalByLocal == maxPreviewLocalByLocal) && minPreviewLocalByLocal == -1)) {
			g2.setColor(Color.BLUE);
			g2.setStroke(new BasicStroke(1.0f));

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			AlphaComposite newAC = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, (float) .15);
			g2.setComposite(newAC);

			//draw preview rectangle...
			Rectangle zoomPreviewRect = new Rectangle();
			zoomPreviewRect.setLocation(theWindow
					.getXCoord(minPreviewLocalByLocal), theWindow
					.getDrawRegion().y - 1);
			zoomPreviewRect.setSize(theWindow.getXCoord(maxPreviewLocalByLocal)
					- (int) zoomPreviewRect.getX(),
					theWindow.getDrawRegion().height + 2);
			g2.fill(zoomPreviewRect);
			g2.setComposite(oldAC);

			g2.drawLine(zoomPreviewRect.x, zoomPreviewRect.y,
					zoomPreviewRect.x, (int) zoomPreviewRect.getMaxY());
			g2.drawLine((int) zoomPreviewRect.getMaxX(), zoomPreviewRect.y,
					(int) zoomPreviewRect.getMaxX(), (int) zoomPreviewRect
							.getMaxY());
		}

		//are we previewing a local zoom by global action?
		if (!((minPreviewLocalByGlobal == maxPreviewLocalByGlobal) && minPreviewLocalByGlobal == -1)) {
			g2.setColor(Color.BLUE);

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			AlphaComposite newAC = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, (float) .15);
			g2.setComposite(newAC);

			//draw preview rectangle...
			g2.fill(theWindow.getDrawRegion());
			g2.setComposite(oldAC);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewLocalTimes(long, long)
	 */
	public void previewLocalTimesByLocal(long minTime, long maxTime) {
		minPreviewLocalByLocal = minTime;
		maxPreviewLocalByLocal = maxTime;
		super.previewLocalTimesByLocal(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewSelectionTimes(long, long)
	 */
	public void previewSelectionTimes(long minTime, long maxTime) {
		minPreviewSel = minTime;
		maxPreviewSel = maxTime;
		super.previewSelectionTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setSelectionTimes(long, long)
	 */
	public void setSelectionTimes(long minTime, long maxTime) {
		minSelectionTime = minTime;
		maxSelectionTime = maxTime;

		super.setSelectionTimes(minTime, maxTime);
	}
}