/*
 * Created on Jul 24, 2004 VisWindow.java
 * 
 * Project: eyetrack_cvs
 */
package view;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;

public abstract class VisWindow {

	/**
	 * method sets the viewing extent of the visualization window
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public abstract void setViewTimes(long minTime, long maxTime);

	/**
	 * method returns the min view time
	 * 
	 * @return
	 */
	public abstract long getMinViewTime();

	/**
	 * method returns the max view time
	 * 
	 * @return
	 */
	public abstract long getMaxViewTime();

	/**
	 * method sets the drawing region for this view
	 * 
	 * @param drawRegion
	 */
	public abstract void setDrawRegion(Rectangle drawRegion);

	/**
	 * method returns the drawing region for this view
	 * 
	 * @return
	 */
	public abstract Rectangle getDrawRegion();

	/**
	 * method draws the window
	 * 
	 * @param g2
	 */
	public abstract void drawWindow(Graphics2D g2);

	/**
	 * method draws the labels for this vis window
	 * 
	 * @param g2
	 * @param f
	 */
	public void drawLabels(Graphics2D g2, Font f) {
	}

	/**
	 * method sets the local view for this vis window
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void setLocalTimes(long minTime, long maxTime) {
	}

	/**
	 * method previews the local view for this vis window by local interaction
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewLocalTimesByLocal(long minTime, long maxTime) {
	}

	/**
	 * method previews the local view for this vis window by global interaction
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewLocalTimesByGlobal(long minTime, long maxTime) {
	}

	/**
	 * method sets the global times for this vis window
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void setGlobalTimes(long minTime, long maxTime) {
	}

	/**
	 * method previews the global times for this vis window
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewGlobalTimes(long minTime, long maxTime) {
	}

	/**
	 * method sets the selection times for this vis window
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void setSelectionTimes(long minTime, long maxTime) {
	}

	/**
	 * method previews the selection times for this vis window
	 * 
	 * @param minTime
	 * @param maxTime
	 */
	public void previewSelectionTimes(long minTime, long maxTime) {
	}

	public abstract double getTime(int x);

	public String getToolTip(java.awt.event.MouseEvent e) {
		return null;
	}

	public abstract int getXCoord(double time);
}