/*
 * Created on Oct 31, 2004
 * GlobalVisWindowCompositor.java
 * 
 * Project: eyetrack_cvs
 */
package view;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

/**
 * @author chris class is meant to composite a top and bottom global view. This
 *         class is most useful when used with a viswindow decorator
 */
public class GlobalVisWindowCompositor extends VisWindow {

	/** our visWindows* */
	protected VisWindow top;

	protected VisWindow bottom;

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewLocalTimesByGlobal(long, long)
	 */
	public void previewLocalTimesByGlobal(long minTime, long maxTime) {
		top.previewLocalTimesByGlobal(minTime, maxTime);
		bottom.previewLocalTimesByGlobal(minTime, maxTime);
	}

	/**
	 * constructor. takes a parameters the windows to composite
	 * 
	 * @param top
	 * @param bottom
	 */
	public GlobalVisWindowCompositor(VisWindow top, VisWindow bottom) {
		this.top = top;
		this.bottom = bottom;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setViewTimes(long, long)
	 */
	public void setViewTimes(long minTime, long maxTime) {
		top.setViewTimes(minTime, maxTime);
		bottom.setViewTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getMinViewTime()
	 */
	public long getMinViewTime() {
		//shouldn't matter which...
		return top.getMinViewTime();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getMaxViewTime()
	 */
	public long getMaxViewTime() {
		//shouldn't matter which...
		return top.getMaxViewTime();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setDrawRegion(java.awt.Rectangle)
	 */
	public void setDrawRegion(Rectangle drawRegion) {
		//as of now, this method does nothing.
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getDrawRegion()
	 */
	public Rectangle getDrawRegion() {
		//return a composited drawing region...
		return top.getDrawRegion().union(bottom.getDrawRegion());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#drawWindow(java.awt.Graphics2D)
	 */
	public void drawWindow(Graphics2D g2) {
		top.drawWindow(g2);
		bottom.drawWindow(g2);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getTime(int)
	 */
	public double getTime(int x) {
		//doesn't matter which
		return top.getTime(x);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getXCoord(double)
	 */
	public int getXCoord(double time) {
		// doesn't matter which
		return top.getXCoord(time);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#drawLabels(java.awt.Graphics2D, java.awt.Font)
	 */
	public void drawLabels(Graphics2D g2, Font f) {
		top.drawLabels(g2, f);
		bottom.drawLabels(g2, f);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#getToolTip(java.awt.event.MouseEvent)
	 */
	public String getToolTip(MouseEvent e) {
		if (top.getDrawRegion().contains(e.getPoint()))
			return top.getToolTip(e);
		else
			return bottom.getToolTip(e);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewGlobalTimes(long, long)
	 */
	public void previewGlobalTimes(long minTime, long maxTime) {
		top.previewGlobalTimes(minTime, maxTime);
		bottom.previewGlobalTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewLocalTimes(long, long)
	 */
	public void previewLocalTimesByLocal(long minTime, long maxTime) {
		top.previewLocalTimesByLocal(minTime, maxTime);
		bottom.previewLocalTimesByLocal(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewSelectionTimes(long, long)
	 */
	public void previewSelectionTimes(long minTime, long maxTime) {
		top.previewSelectionTimes(minTime, maxTime);
		bottom.previewSelectionTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setGlobalTimes(long, long)
	 */
	public void setGlobalTimes(long minTime, long maxTime) {
		top.setGlobalTimes(minTime, maxTime);
		bottom.setGlobalTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setLocalTimes(long, long)
	 */
	public void setLocalTimes(long minTime, long maxTime) {
		top.setLocalTimes(minTime, maxTime);
		bottom.setLocalTimes(minTime, maxTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setSelectionTimes(long, long)
	 */
	public void setSelectionTimes(long minTime, long maxTime) {
		top.setSelectionTimes(minTime, maxTime);
		bottom.setSelectionTimes(minTime, maxTime);
	}
}