/*
 * Created on Oct 31, 2004
 * GlobalWindowDecorator.java
 * 
 * Project: eyetrack_cvs
 */
package view;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;

/**
 * @author chris
 *  
 */
public class GlobalWindowDecorator extends VisWindowDecorator {

	/** the min preview selection time* */
	private long minPreviewSel = -1;

	/** the max preview selection time* */
	private long maxPreviewSel = -1;

	/** the min local preview time by local action* */
	private long minPreviewLocalByLocal = -1;

	/** the max local preview time by local action* */
	private long maxPreviewLocalByLocal = -1;

	/** the min local preview time by global action */
	private long minPreviewLocalByGlobal = -1;

	/** the max local preview time by gobal action */
	private long maxPreviewLocalByGlobal = -1;

	/** the min selection time* */
	private long minSelectionTime = -1;

	/** the max selection time* */
	private long maxSelectionTime = -1;

	/** the local min time */
	private long minLocalTime = -1;

	/** the local max time */
	private long maxLocalTime = -1;

	/**
	 * @param theWindow
	 */
	public GlobalWindowDecorator(VisWindow theWindow) {
		super(theWindow);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#drawWindow(java.awt.Graphics2D)
	 */
	public void drawWindow(Graphics2D g2) {
		super.drawWindow(g2);

		//are we presently displaying a selection?
		if (!((minSelectionTime == maxSelectionTime) && minSelectionTime == -1)) {
			g2.setColor(Color.LIGHT_GRAY);
			Rectangle selRect = new Rectangle();

			//draw the selection slightly lighter if we are currently
			//setting a selection
			float alpha = .35f;
			if (!((minPreviewSel == maxPreviewSel) && minPreviewSel == -1))
				alpha = .15f;

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					alpha));

			if (theWindow.getXCoord(minSelectionTime) < theWindow
					.getDrawRegion().x)
				selRect.setLocation(theWindow.getDrawRegion().x, theWindow
						.getDrawRegion().y - 1);
			else
				selRect.setLocation(theWindow.getXCoord(minSelectionTime),
						theWindow.getDrawRegion().y - 1);

			if (theWindow.getXCoord(maxSelectionTime) > theWindow
					.getDrawRegion().getMaxX())
				selRect.setSize((int) theWindow.getDrawRegion().getMaxX()
						- (int) selRect.getX(),
						theWindow.getDrawRegion().height + 2);
			else
				selRect.setSize(theWindow.getXCoord(maxSelectionTime)
						- (int) selRect.getX(),
						theWindow.getDrawRegion().height + 2);

			//draw onto the screen
			g2.fill(selRect);

			//reset the composite
			g2.setComposite(oldAC);
		}

		//are we previewing a selection?
		if (!((minPreviewSel == maxPreviewSel) && minPreviewSel == -1)) {
			g2.setColor(Color.LIGHT_GRAY);
			Rectangle selRect = new Rectangle();

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					.35f));

			if (theWindow.getXCoord(minPreviewSel) < theWindow.getDrawRegion().x)
				selRect.setLocation(theWindow.getDrawRegion().x, theWindow
						.getDrawRegion().y - 1);
			else
				selRect.setLocation(theWindow.getXCoord(minPreviewSel),
						theWindow.getDrawRegion().y - 1);

			if (theWindow.getXCoord(maxPreviewSel) > theWindow.getDrawRegion()
					.getMaxX())
				selRect.setSize((int) theWindow.getDrawRegion().getMaxX()
						- (int) selRect.getX(),
						theWindow.getDrawRegion().height + 2);
			else
				selRect.setSize(theWindow.getXCoord(maxPreviewSel)
						- (int) selRect.getX(),
						theWindow.getDrawRegion().height + 2);

			//draw onto the screen
			g2.fill(selRect);

			//reset the composite
			g2.setComposite(oldAC);
		}

		//are we previewing a global interaction?
		if (!((minPreviewLocalByGlobal == maxPreviewLocalByGlobal) && minPreviewLocalByGlobal == -1)) {
			g2.setColor(Color.BLUE);
			g2.setStroke(new BasicStroke(3.0f));

			Rectangle magRectangle = new Rectangle();

			if (theWindow.getXCoord(minPreviewLocalByGlobal) < theWindow
					.getDrawRegion().x)
				magRectangle.setLocation(theWindow.getDrawRegion().x, theWindow
						.getDrawRegion().y);
			else
				magRectangle.setLocation(theWindow
						.getXCoord(minPreviewLocalByGlobal), theWindow
						.getDrawRegion().y);

			if (theWindow.getXCoord(maxPreviewLocalByGlobal) > theWindow
					.getDrawRegion().getMaxX())
				magRectangle.setSize((int) theWindow.getDrawRegion().getMaxX()
						- (int) magRectangle.getX(),
						theWindow.getDrawRegion().height + 1);
			else
				magRectangle.setSize(theWindow
						.getXCoord(maxPreviewLocalByGlobal)
						- (int) magRectangle.getX(),
						theWindow.getDrawRegion().height + 1);

			g2.draw(magRectangle);

			//draw the old local view...
			if (!((minLocalTime == maxLocalTime) && minLocalTime == -1)) {
				AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
				g2.setComposite(AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, .15f));

				if (theWindow.getXCoord(minLocalTime) < theWindow
						.getDrawRegion().x)
					magRectangle.setLocation(theWindow.getDrawRegion().x,
							theWindow.getDrawRegion().y - 1);
				else
					magRectangle.setLocation(theWindow.getXCoord(minLocalTime),
							theWindow.getDrawRegion().y - 1);

				if (theWindow.getXCoord(maxLocalTime) > theWindow
						.getDrawRegion().getMaxX())
					magRectangle.setSize((int) theWindow.getDrawRegion()
							.getMaxX()
							- (int) magRectangle.getX(), theWindow
							.getDrawRegion().height + 2);
				else
					magRectangle.setSize(theWindow.getXCoord(maxLocalTime)
							- (int) magRectangle.getX(), theWindow
							.getDrawRegion().height + 2);

				//draw onto the screen
				g2.fill(magRectangle);

				//reset the composite
				g2.setComposite(oldAC);
			}
			return;
		}

		//are we previewing a local interaction?
		if (!((minPreviewLocalByLocal == maxPreviewLocalByLocal) && minPreviewLocalByLocal == -1)) {

			g2.setColor(Color.BLUE);
			g2.setStroke(new BasicStroke(3.0f));
			Rectangle magRectangle = new Rectangle();

			AlphaComposite oldAC = (AlphaComposite) g2.getComposite();
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
					.15f));

			if (theWindow.getXCoord(minPreviewLocalByLocal) < theWindow
					.getDrawRegion().x)
				magRectangle.setLocation(theWindow.getDrawRegion().x, theWindow
						.getDrawRegion().y - 1);
			else
				magRectangle.setLocation(theWindow
						.getXCoord(minPreviewLocalByLocal), theWindow
						.getDrawRegion().y - 1);

			if (theWindow.getXCoord(maxPreviewLocalByLocal) > theWindow
					.getDrawRegion().getMaxX())
				magRectangle.setSize((int) theWindow.getDrawRegion().getMaxX()
						- (int) magRectangle.getX(),
						theWindow.getDrawRegion().height + 2);
			else
				magRectangle.setSize(theWindow
						.getXCoord(maxPreviewLocalByLocal)
						- (int) magRectangle.getX(),
						theWindow.getDrawRegion().height + 2);

			//draw onto the screen
			g2.fill(magRectangle);

			//reset the composite
			g2.setComposite(oldAC);

			//draw the magnification window?
			if (!((minLocalTime == maxLocalTime) && minLocalTime == -1)) {

				if (theWindow.getXCoord(minLocalTime) < theWindow
						.getDrawRegion().x)
					magRectangle.setLocation(theWindow.getDrawRegion().x,
							theWindow.getDrawRegion().y);
				else
					magRectangle.setLocation(theWindow.getXCoord(minLocalTime),
							theWindow.getDrawRegion().y);

				if (theWindow.getXCoord(maxLocalTime) > theWindow
						.getDrawRegion().getMaxX())
					magRectangle.setSize((int) theWindow.getDrawRegion()
							.getMaxX()
							- (int) magRectangle.getX(), theWindow
							.getDrawRegion().height + 1);
				else
					magRectangle.setSize(theWindow.getXCoord(maxLocalTime)
							- (int) magRectangle.getX(), theWindow
							.getDrawRegion().height + 1);

				g2.draw(magRectangle);
			}
			return;
		}

		//draw the magnification window?
		if (!((minLocalTime == maxLocalTime) && minLocalTime == -1)) {
			g2.setColor(Color.BLUE);
			g2.setStroke(new BasicStroke(3.0f));
			Rectangle magRectangle = new Rectangle();

			if (theWindow.getXCoord(minLocalTime) < theWindow.getDrawRegion().x)
				magRectangle.setLocation(theWindow.getDrawRegion().x, theWindow
						.getDrawRegion().y);
			else
				magRectangle.setLocation(theWindow.getXCoord(minLocalTime),
						theWindow.getDrawRegion().y);

			if (theWindow.getXCoord(maxLocalTime) > theWindow.getDrawRegion()
					.getMaxX())
				magRectangle.setSize((int) theWindow.getDrawRegion().getMaxX()
						- (int) magRectangle.getX(),
						theWindow.getDrawRegion().height + 1);
			else
				magRectangle.setSize(theWindow.getXCoord(maxLocalTime)
						- (int) magRectangle.getX(),
						theWindow.getDrawRegion().height + 1);

			g2.draw(magRectangle);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewLocalTimes(long, long)
	 */
	public void previewLocalTimesByLocal(long minTime, long maxTime) {
		super.previewLocalTimesByLocal(minTime, maxTime);
		minPreviewLocalByLocal = minTime;
		maxPreviewLocalByLocal = maxTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewSelectionTimes(long, long)
	 */
	public void previewSelectionTimes(long minTime, long maxTime) {
		super.previewSelectionTimes(minTime, maxTime);
		minPreviewSel = minTime;
		maxPreviewSel = maxTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setLocalTimes(long, long)
	 */
	public void setLocalTimes(long minTime, long maxTime) {
		super.setLocalTimes(minTime, maxTime);
		minLocalTime = minTime;
		maxLocalTime = maxTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setSelectionTimes(long, long)
	 */
	public void setSelectionTimes(long minTime, long maxTime) {
		super.setSelectionTimes(minTime, maxTime);
		minSelectionTime = minTime;
		maxSelectionTime = maxTime;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewLocalTimesByGlobal(long, long)
	 */
	public void previewLocalTimesByGlobal(long minTime, long maxTime) {
		super.previewLocalTimesByGlobal(minTime, maxTime);
		minPreviewLocalByGlobal = minTime;
		maxPreviewLocalByGlobal = maxTime;
	}
}