/*
 * Created on Jul 27, 2004
 * StatsModule.java
 * 
 * Project: eyetrack_cvs
 */
package view;

import java.awt.Dimension;
import java.awt.Font;
import java.text.DecimalFormat;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import pupil.DataPacket;
import pupil.PupilDataPacket;

/**
 * @author chris
 *  
 */
public class StatsView extends JDialog implements View {

	/** the statistics data* */
	private String[][] data;

	/** static vals representing the rows/cols of data* */
	private static int SEL_ROW = 0;

	private static int LOCAL_ROW = 1;

	private static int GLOBAL_ROW = 2;

	private static int MEAN_COL = 0;

	private static int STDEV_COL = 1;

	private static int MINPUP_COL = 2;

	private static int MAXPUP_COL = 3;

	private static int TIME_COL = 4;

	/** the number formate for displaying floats* */
	private DecimalFormat df = new DecimalFormat("#.##");

	/** the table model* */
	private TheTableModel theTableModel;

	/** the pupil data */
	private Vector pupilData = null;

	/** the task model */
	private Vector taskModel = null;

	/** the baseline */
	private float baseline = 0;

	/** local min time */
	private long localMin;

	/** local max time */
	private long localMax;

	/** local stats vector */
	private Vector localStats;

	/** min seleciton time */
	private long selMin;

	/** max selection time */
	private long selMax;

	/** sel stats vector */
	private Vector selStats;

	/** min global time */
	private long globalMin;

	/** max global time */
	private long globalMax;

	/** global stats vector */
	private Vector globalStats;

	/** the mask threshhold */
	private float maskThresh = -1;

	/**
	 * the table model for the statistics frame
	 * 
	 * @author chris
	 *  
	 */
	private class TheTableModel extends AbstractTableModel {

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getColumnClass(int)
		 */
		public Class getColumnClass(int arg0) {
			return String.class;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getColumnName(int)
		 */
		public String getColumnName(int c) {
			switch (c) {
			case 0:
				return null;
			case 1:
				return "Mean Pupil";
			case 2:
				return "St. Dev.";
			case 3:
				return "Min Pupil at ms";
			case 4:
				return "Max Pupil at ms";
			case 5:
				return "Time Frame (ms)";
			default:
				return null;
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		public int getColumnCount() {
			return 6;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		public int getRowCount() {
			return 3;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int r, int c) {
			if (c == 0) {
				switch (r) {
				case 0:
					return "Selected Frame *";
				case 1:
					return "Detail Frame";
				case 2:
					return "Overview Frame";
				}
				return null;
			}

			//otherwise, return data
			return data[r][c - 1];
		}
	}

	/**
	 * default constructor
	 *  
	 */
	public StatsView(JFrame owner, String title) {
		super(owner, title);

		data = new String[3][5];

		//we're not modal...
		setModal(false);
		this.setResizable(true);

		//initialize the stats vectors
		localStats = new Vector(4);
		selStats = new Vector(4);
		globalStats = new Vector(4);

		//set up the table
		theTableModel = new TheTableModel();
		JTable table = new JTable(theTableModel);
		table.setFont(new Font(table.getFont().getFamily(), Font.PLAIN, table.getFont().getSize() + 2));

		TableColumn column;
		column = table.getColumnModel().getColumn(0);
		column.setPreferredWidth(column.getPreferredWidth() * 2);
		column = table.getColumnModel().getColumn(3);
		column.setPreferredWidth(column.getPreferredWidth() * 2);
		column = table.getColumnModel().getColumn(4);
		column.setPreferredWidth(column.getPreferredWidth() * 2);
		column = table.getColumnModel().getColumn(5);
		column.setPreferredWidth(column.getPreferredWidth() * 2);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		table.getTableHeader().setReorderingAllowed(false);

		//because of some wacky glitch on the mac jre, manually
		//set the table header height
		table.getTableHeader().setPreferredSize(
				new Dimension(table.getTableHeader().getPreferredSize().width,
						20));
		JTableHeader jth = table.getTableHeader();
		jth.setFont(new Font(jth.getFont().getFamily(), Font.BOLD, jth.getFont().getSize() + 2));
 

		JScrollPane scrollPane = new JScrollPane(table);
		table.setPreferredScrollableViewportSize(table.getPreferredSize());
		this.getContentPane().setLayout(
				new BoxLayout(this.getContentPane(), BoxLayout.PAGE_AXIS));
		this.getContentPane().add(new JScrollPane(table));
		JPanel tmpPanel = new JPanel();
		tmpPanel.setLayout(new BoxLayout(tmpPanel, BoxLayout.LINE_AXIS));
		tmpPanel.add(Box.createHorizontalGlue());
		tmpPanel
				.add(new JLabel(
						"* Create Selection with Shift+Time Window Drag in Local Frame"));
		this.getContentPane().add(tmpPanel);
		pack();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setFocus(double, double)
	 */
	public void setLocal(long minTime, long maxTime) {
		if (minTime == maxTime)
			data[LOCAL_ROW][4] = "";
		else
			data[LOCAL_ROW][4] = (long) minTime + " to " + (long) maxTime;

		//check if stats need to be recomputed
		if (localMin != minTime || localMax != maxTime) {
			localMin = (long) minTime;
			localMax = (long) maxTime;
			computeStats(localMin, localMax, localStats);
		}

		//draw stats
		if (localStats.get(0) == null)
			data[LOCAL_ROW][MEAN_COL] = "";
		else
			data[LOCAL_ROW][MEAN_COL] = df.format(localStats.get(0));
		if (localStats.get(1) == null)
			data[LOCAL_ROW][STDEV_COL] = "";
		else
			data[LOCAL_ROW][STDEV_COL] = df.format(localStats.get(1));
		if (localStats.get(2) == null || localStats.get(3) == null)
			data[LOCAL_ROW][MINPUP_COL] = "";
		else
			data[LOCAL_ROW][MINPUP_COL] = df.format(localStats.get(2)) + " at "
					+ localStats.get(3);
		if (localStats.get(4) == null || localStats.get(5) == null)
			data[LOCAL_ROW][MAXPUP_COL] = "";
		else
			data[LOCAL_ROW][MAXPUP_COL] = df.format(localStats.get(4)) + " at "
					+ localStats.get(5);

		//update the table model
		theTableModel.fireTableDataChanged();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewFocus(double, double)
	 */
	public void previewLocal(long minTime, long maxTime) {
		if (minTime == maxTime)
			data[LOCAL_ROW][TIME_COL] = "";
		else
			data[LOCAL_ROW][TIME_COL] = (int) minTime + " to " + (int) maxTime;

		//clear out drawn stats
		for (int c = 0; c < 4; c++)
			data[LOCAL_ROW][c] = "";

		theTableModel.fireTableDataChanged();
	}

	/**
	 *  
	 */
	public void setSelection(long minTime, long maxTime) {
		if (minTime == maxTime)
			data[SEL_ROW][TIME_COL] = "";
		else {
			data[SEL_ROW][TIME_COL] = (long) minTime + " to " + (long) maxTime;
		}

		//do stats need to be recomputed?
		if (minTime != selMin || maxTime != selMax) {
			selMin = (long) minTime;
			selMax = (long) maxTime;
			computeStats(selMin, selMax, selStats);
		}

		//draw stats
		if (selStats.get(0) == null)
			data[SEL_ROW][MEAN_COL] = "";
		else
			data[SEL_ROW][MEAN_COL] = df.format(selStats.get(0));
		if (selStats.get(1) == null)
			data[SEL_ROW][STDEV_COL] = "";
		else
			data[SEL_ROW][STDEV_COL] = df.format(selStats.get(1));
		if (selStats.get(2) == null || selStats.get(3) == null)
			data[SEL_ROW][MINPUP_COL] = "";
		else
			data[SEL_ROW][MINPUP_COL] = df.format(selStats.get(2)) + " at "
					+ selStats.get(3);
		if (selStats.get(4) == null || selStats.get(5) == null)
			data[SEL_ROW][MAXPUP_COL] = "";
		else
			data[SEL_ROW][MAXPUP_COL] = df.format(selStats.get(4)) + " at "
					+ selStats.get(5);

		theTableModel.fireTableDataChanged();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#previewSelection(double, double)
	 */
	public void previewSelection(long minTime, long maxTime) {
		if (minTime == maxTime)
			data[SEL_ROW][TIME_COL] = "";
		else
			data[SEL_ROW][TIME_COL] = (int) minTime + " to " + (int) maxTime;

		//clear out drawn stats
		for (int c = 0; c < 4; c++)
			data[SEL_ROW][c] = "";

		theTableModel.fireTableDataChanged();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see view.VisWindow#setContext(double, double)
	 */
	public void setGlobal(long minTime, long maxTime) {

		if (minTime == maxTime)
			data[GLOBAL_ROW][TIME_COL] = "";
		else
			data[GLOBAL_ROW][TIME_COL] = (int) minTime + " to " + (int) maxTime;

		//compute new stats?
		if (minTime != globalMin || maxTime != globalMax) {
			globalMin = (long) minTime;
			globalMax = (long) maxTime;
			computeStats(globalMin, globalMax, globalStats);
		}

		//draw stats
		if (globalStats.get(0) == null)
			data[GLOBAL_ROW][MEAN_COL] = "";
		else
			data[GLOBAL_ROW][MEAN_COL] = df.format(globalStats.get(0));
		if (globalStats.get(1) == null)
			data[GLOBAL_ROW][STDEV_COL] = "";
		else
			data[GLOBAL_ROW][STDEV_COL] = df.format(globalStats.get(1));
		if (globalStats.get(2) == null || globalStats.get(3) == null)
			data[GLOBAL_ROW][MINPUP_COL] = "";
		else
			data[GLOBAL_ROW][MINPUP_COL] = df.format(globalStats.get(2))
					+ " at " + globalStats.get(3);
		if (globalStats.get(4) == null || globalStats.get(5) == null)
			data[GLOBAL_ROW][MAXPUP_COL] = "";
		else
			data[GLOBAL_ROW][MAXPUP_COL] = df.format(globalStats.get(4))
					+ " at " + globalStats.get(5);
	}

	/**
	 * given min and max times, method computes statistics and stores them in
	 * the stats vector
	 * 
	 * @param minTime
	 *            min time
	 * @param maxTime
	 *            max time
	 * @param stats
	 *            the stats vector
	 */
	private void computeStats(long minTime, long maxTime, Vector stats) {
		//pass in null if no stats exist...
		if (pupilData == null || (minTime == maxTime)) {
			stats.clear();
			for (int i = 0; i < 6; i++) {
				stats.add(null);
			}
		} else {
			//get the min and max data packets for this selection window
			int minIndex, maxIndex;
			for (minIndex = 0; ((DataPacket) pupilData.get(minIndex))
					.getTimeStamp() < minTime; minIndex++) {
			}
			for (maxIndex = minIndex; (maxIndex < pupilData.size() && ((DataPacket) pupilData
					.get(maxIndex)).getTimeStamp() <= maxTime); maxIndex++) {
			}
			if (maxIndex != minIndex)
				maxIndex--;

			//compute stats
			float mean = 0;
			float minSize = Float.MAX_VALUE;
			long minSizeTime = 0;
			float maxSize = Float.MIN_VALUE;
			long maxSizeTime = 0;
			int count = 0;
			float stdev = 0;

			//compute mean, max, min, count
			for (int i = minIndex; i <= maxIndex; i++) {
				if (pupilData.get(i) instanceof PupilDataPacket) {
					PupilDataPacket DP = (PupilDataPacket) pupilData.get(i);
					if ((DP.getPupilSize() - baseline) / baseline < maskThresh)
						continue;

					mean += DP.getPupilSize();
					count++;
					if (minSize > DP.getPupilSize()) {
						minSize = DP.getPupilSize();
						minSizeTime = DP.getTimeStamp();
					}
					if (maxSize < DP.getPupilSize()) {
						maxSize = DP.getPupilSize();
						maxSizeTime = DP.getTimeStamp();
					}
				}
			}

			if (count == 0)
				return;

			mean /= count;
			mean = (mean - baseline) / baseline;

			//compute the st. dev.
			for (int i = minIndex; i <= maxIndex; i++) {
				if (pupilData.get(i) instanceof PupilDataPacket) {
					PupilDataPacket DP = (PupilDataPacket) pupilData.get(i);
					if ((DP.getPupilSize() - baseline) / baseline < maskThresh)
						continue;
					stdev += (mean - (DP.getPupilSize() - baseline) / baseline)
							* (mean - (DP.getPupilSize() - baseline) / baseline);
				}
			}
			stdev /= count;

			minSize = 100 * (minSize - baseline) / baseline;
			maxSize = 100 * (maxSize - baseline) / baseline;
			mean = 100 * mean;
			stdev = 100 * (float) Math.sqrt(stdev);

			//add stats to the stats vector
			stats.clear();
			stats.add(new Float(mean));
			stats.add(new Float(stdev));
			stats.add(new Float(minSize));
			stats.add(new Long(minSizeTime));
			stats.add(new Float(maxSize));
			stats.add(new Long(maxSizeTime));
		}

	}

	/**
	 * @return Returns the pupilData.
	 */
	public Vector getPupilData() {
		return pupilData;
	}

	/**
	 * @param pupilData
	 *            The pupilData to set.
	 */
	public void setPupilData(Vector pupilData) {
		this.pupilData = pupilData;
	}

	/**
	 * @return Returns the baseline.
	 */
	public float getBaseline() {
		return baseline;
	}

	/**
	 * @param baseline
	 *            The baseline to set.
	 */
	public void setBaseline(float baseline) {
		this.baseline = baseline;
	}

	/**
	 * @return Returns the taskModel.
	 */
	public Vector getTaskModel() {
		return taskModel;
	}

	/**
	 * @param taskModel
	 *            The taskModel to set.
	 */
	public void setTaskModel(Vector taskModel) {
		this.taskModel = taskModel;
	}

	/**
	 * method removes all content from the stats panel
	 *  
	 */
	public void purge() {
		taskModel = null;
		pupilData = null;
		setLocal(-1, -1);
		setGlobal(-1, -1);
		setSelection(-1, -1);
	}

	/**
	 * method sets the masking threshhold
	 * 
	 * @param maskThresh
	 */
	public void setMaskThreshold(float maskThresh) {
		this.maskThresh = maskThresh;

		//recompute local stats?
		if (localMin != localMax) {
			computeStats(localMin, localMax, localStats);

			//draw stats
			if (localStats.get(0) == null)
				data[LOCAL_ROW][MEAN_COL] = "";
			else
				data[LOCAL_ROW][MEAN_COL] = df.format(localStats.get(0));
			if (localStats.get(1) == null)
				data[LOCAL_ROW][STDEV_COL] = "";
			else
				data[LOCAL_ROW][STDEV_COL] = df.format(localStats.get(1));
			if (localStats.get(2) == null || localStats.get(3) == null)
				data[LOCAL_ROW][MINPUP_COL] = "";
			else
				data[LOCAL_ROW][MINPUP_COL] = df.format(localStats.get(2))
						+ " at " + localStats.get(3);
			if (localStats.get(4) == null || localStats.get(5) == null)
				data[LOCAL_ROW][MAXPUP_COL] = "";
			else
				data[LOCAL_ROW][MAXPUP_COL] = df.format(localStats.get(4))
						+ " at " + localStats.get(5);
		}

		//recompute selection stats?
		if (selMin != selMax) {
			computeStats(selMin, selMax, selStats);

			//draw stats
			if (selStats.get(0) == null)
				data[SEL_ROW][MEAN_COL] = "";
			else
				data[SEL_ROW][MEAN_COL] = df.format(selStats.get(0));
			if (selStats.get(1) == null)
				data[SEL_ROW][STDEV_COL] = "";
			else
				data[SEL_ROW][STDEV_COL] = df.format(selStats.get(1));
			if (selStats.get(2) == null || selStats.get(3) == null)
				data[SEL_ROW][MINPUP_COL] = "";
			else
				data[SEL_ROW][MINPUP_COL] = df.format(selStats.get(2)) + " at "
						+ selStats.get(3);
			if (selStats.get(4) == null || selStats.get(5) == null)
				data[SEL_ROW][MAXPUP_COL] = "";
			else
				data[SEL_ROW][MAXPUP_COL] = df.format(selStats.get(4)) + " at "
						+ selStats.get(5);
		}

		//recompute global stats
		if (globalMin != globalMax) {
			computeStats(globalMin, globalMax, globalStats);

			//draw stats
			if (globalStats.get(0) == null)
				data[GLOBAL_ROW][MEAN_COL] = "";
			else
				data[GLOBAL_ROW][MEAN_COL] = df.format(globalStats.get(0));
			if (globalStats.get(1) == null)
				data[GLOBAL_ROW][STDEV_COL] = "";
			else
				data[GLOBAL_ROW][STDEV_COL] = df.format(globalStats.get(1));
			if (globalStats.get(2) == null || globalStats.get(3) == null)
				data[GLOBAL_ROW][MINPUP_COL] = "";
			else
				data[GLOBAL_ROW][MINPUP_COL] = df.format(globalStats.get(2))
						+ " at " + globalStats.get(3);
			if (globalStats.get(4) == null || globalStats.get(5) == null)
				data[GLOBAL_ROW][MAXPUP_COL] = "";
			else
				data[GLOBAL_ROW][MAXPUP_COL] = df.format(globalStats.get(4))
						+ " at " + globalStats.get(5);
		}

		//update the table model
		theTableModel.fireTableDataChanged();
	}

}